/***************************************************************************************************
 *
 *  CTT3. Original versions by VUC++. Version 3 by Zeropoint.
 *
 *  $CLASS        CTTXmasTree
 *  $VERSION      1.03 (2-12-2007 0:24)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  The xmas tree replacement for the CTF flags.
 *
 **************************************************************************************************/ 
class CTTXmasTree extends CTFFlag;

var() class<CTTXmasTreeLight> lightClass;         // Class of the lights blinking on the tree.

var Pawn xHolder;                                 // The pawn that is holding the flag.
var PlayerPawn localPlayer;                       // Player of the local client.

const minRadius = 112.0;                          // Minimum light spawn distance from origin.
const maxRadius = 128.0;                          // Maximum light spawn distance from origin.
const carriedFlagHOffset = -32.0;                 // Horizontal tree offset relative to the tree holder.
const carriedFlagVOffset = 32.0;                  // Vertical tree offset relative to the tree holder.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {
    reliable if (role == ROLE_Authority && bHeld)
        xHolder;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the Actor is spawned. Initializes the flag... err tree.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function postBeginPlay() {
	super(Decoration).postBeginPlay();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Makes the tree blink.
 *
 **************************************************************************************************/
simulated function blink() {
	local vector spawnLocation;
	local float hAngle;
	local float vAngle;
	local float radius;
	local CTTXmasTreeLight light;

	// Get origin of the flag.
	if (bHeld) {
		if (xHolder == none || xHolder == localPlayer) return;
		hAngle = xHolder.rotation.yaw / 32768.0 * pi;
		spawnLocation = xHolder.location;
		spawnLocation.x += cos(hAngle) * carriedFlagHOffset;
		spawnLocation.y += sin(hAngle) * carriedFlagHOffset;
		spawnLocation.z += carriedFlagVOffset;
	} else {
		spawnLocation = self.location;
	}
	
	// Determine a random spawn location around the origin.
	hAngle = fRand() * 2 * pi;
	vAngle = fRand() * 2 * pi;
	radius = (fRand() * (maxRadius - minRadius) - minRadius);
	if (bHome) {
		radius *= 0.625;
	} else if (bHeld) {
		radius *= 0.2;
	} else {
		radius *= 0.31;
	}
	spawnLocation.x += radius * sin(vAngle) * cos(hAngle);
	spawnLocation.y += radius * sin(vAngle) * sin(hAngle);
	spawnLocation.z += radius * cos(vAngle);
	
	// Spawn the light.
	light = spawn(lightClass, , , spawnLocation);
	
	// Modify light attributes if the tree is held.
	if (bHeld) {
		light.setBase(xHolder);
		light.drawScale *= 0.5;
	} else {
		light.setBase(self);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  State in which the tree is held by a player.
 *
 **************************************************************************************************/
state Held {
	
	
	
	/***********************************************************************************************
	 *
	 *  $DESCRIPTION  Called when the tree fell out of the world.
	 *  $OVERRIDE
	 *
	 **********************************************************************************************/
	event fellOutOfWorld() {
		// Doesn't make sense while it is carried.
	}



	/***********************************************************************************************
	 *
	 *  $DESCRIPTION  Timer tick.
	 *  $OVERRIDE
	 *
	 **********************************************************************************************/
	function timer() {
		if (holder == none) {
			sendHome();
		}
	}



	/***********************************************************************************************
	 *
	 *  $DESCRIPTION  Called when the held state was entered.
	 *  $OVERRIDE
	 *
	 **********************************************************************************************/
	function beginState() {
		bHeld = true;
		bCollideWorld = false;
		bKnownLocation = false;
		homeBase.playAlarm();
		setPhysics(PHYS_None);
		setCollision(false, false, false);
		setTimer(10.0, true);
		drawScale = 0.4;
		xHolder = holder;
	}



	/***********************************************************************************************
	 *
	 *  $DESCRIPTION  Called when the held state was left.
	 *  $OVERRIDE
	 *
	 **********************************************************************************************/
	function endState() {
		bHeld = false;
		drawScale = 0.6;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
    lightClass=class'CTTXmasTreeLight'
}

