/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenConfig
 *  $VERSION      1.04 (27-1-2007 12:28)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Nexgen configuration container/replication class. This class contains the settings
 *                for the server controller and necessary code to gather a lot of required data.
 *
 **************************************************************************************************/
class NexgenConfig extends ReplicationInfo config(Nexgen);

var bool bInitialized;                            // NexgenConfig instance has been initialized.

var NexgenController control;                     // Server controller.
var int mapEntriesPerGame;                        // Amount of entries available in map list for
                                                  // each game configuration.
var string games[16];                             // List of all available game types on the server.
var string maps[32];                              // List of all available maps on the server.
                                                  // NOTE: This is a composite array list.
var string mutators[48];                          // List of all available mutators on the server.

const maxListEntrySize = 250;                     // Max characters allowed in a single entry for a
                                                  // composite array list.
const separator = ",";                            // Character used to seperate elements in a list.

var string serverID;                              // Public (unique) server identification code.
                                                  // Should be the last value replicated, so the
                                                  // client can check when the replication is
                                                  // complete.

// Special settings.
var config bool bInstalled;                       // Whether or not Nexgen has been installed.
var private config string serverKey;              // Unique server key.

// Global server settings.
var config string serverName;                     // Name of the server.
var config string shortName;                      // Short server name.
var config string adminName;                      // Name of the admin that controls the server.
var config string adminEmail;                     // Email of the admin that controls the server.
var config string serverPassword;                 // Password needed to enter the server.
var config bool spectatorsNeedPassword;           // Do spectators need to enter the password?
var config int playerSlots;                       // Maximum number of players.
var config int vipSlots;                          // Extra slots for VIPs if the server if full.
var config int adminSlots;                        // Extra slots for admins if the server if full.
var config int spectatorSlots;                    // Number of spectators allowed.

// Extended server settings.
var config bool autoUpdateBans;                   // Automatically update bans?
var config int waitTime;                          // Time to wait before the game can be started.
var config int startTime;                         // Time to wait before the game starts.

// Extended HUD.
var config bool useNexgenMessageHUD;              // Enable the Nexgen message HUD replacement.
var config string replacementClass[16];           // HUD class replacement instructions.

// Game vote config.
var config string gcGameClass[10];                // Game class.
var config string gcGameName[10];                 // Game name.
var config string gcMutatorClasses[10];           // Mutator classes to use for this game.
var config string gcMutatorNames[10];             // Names of the mutators to use.
var config string gcGameMaps[250];                // Map list for game vote configurations. This
                                                  // NOTE: This is a composite array list.
var config int scoreboardTime;                    // Display the scoreboard this many seconds.
var config int voteTime;                          // Time available to cast a vote (in seconds).

// Account system.
var config string atTypeName[10];                 // Account type names.
var config string atRights[10];                   // Rights for the account types.
var config string atTitle[10];                    // Titles for the account types.
var config string paPlayerID[32];                 // Player ID list.
var config string paPlayerName[32];               // Names of the player accounts.
var config int paAccountType[32];                 // Associated account type.
var config string paCustomRights[32];             // Custom rights if no account type is used.
var config string paCustomTitle[32];              // Custom title if no account type is used.

// Ban system.
var config string bannedName[32];                 // Name of banned player.
var config string bannedIPs[32];                  // IP address(es) of the banned player.
var config string bannedIDs[32];                  // Client ID(s) of the banned player.
var config string banReason[32];                  // Reason why the player was banned.
var config string banPeriod[32];                  // Ban period; how long the player is banned.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {

	reliable if (role == ROLE_Authority)
	    // Dynamic.
		gcGameClass, gcGameName, gcMutatorClasses, gcMutatorNames, gcGameMaps,
		
		serverName, shortName, adminName, adminEmail, serverPassword, playerSlots, vipSlots,
		adminSlots, spectatorSlots,

		// Semi static.
		games, maps, mutators, serverID;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads the server configuration.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function preBeginPlay() {
	
	// Check owner.
	if (owner == none || !owner.isA('NexgenController')) {
		destroy();
		return;
	}
	control = NexgenController(owner);
	
	// Setup variables.
	mapEntriesPerGame = ArrayCount(gcGameMaps) / ArrayCount(gcGameClass);
	
	// Load game types, maps and mutators for voting config.
	loadGameTypeList();
	loadMapList();
	loadMutatorList();
	
	// Finish initialization.
	bInitialized = true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Finalizes the initialization process. Calling this function makes sure that any
 *                uninitialized variables are set.
 *  $REQUIRE      bInitialized && bInstalled
 *
 **************************************************************************************************/
function postInitialize() {
	serverID = class'MD5Hash'.static.MD5String(serverKey);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads a list of all game types available on this server.
 *  $REQUIRE      !bInitialized
 *
 **************************************************************************************************/
function loadGameTypeList() {
	local string game;
	local string description;
	local int index;
	local string gameInfoStr;
	local class<GameInfo> gameClass;
	local int gameListIndex;
	local string mapName;
	
	// For each game type...
	getNextIntDesc("TournamentGameInfo", 0, game, description);
	while(game != "") {
		
		// Get game type info.
		gameClass = class<GameInfo>(dynamicLoadObject(game, class'Class'));
		
		gameInfoStr = game $ separator $
		              gameClass.default.gameName $ separator $
		              gameClass.default.mapPrefix;
		
		// Store game type info.
		if (gameListIndex >= arrayCount(games)) {
			control.nscLog("Insufficient space available for storing game type info.");
			return;
		} else {
			games[gameListIndex] = gameInfoStr;
			gameListIndex++;
		}
		
		// Continue with next game type.
		getNextIntDesc("TournamentGameInfo", ++index, game, description);
	}	
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads a list of all maps available on this server.
 *  $REQUIRE      !bInitialized
 *
 **************************************************************************************************/
function loadMapList() {
	local string mapName;
	local string firstMap;
	local int mapListIndex;
	local string shortMapName;
	local bool bFirst;
	
	// For each map...
	mapName = getMapName("", "", 0);
	firstMap = mapName;
	bFirst = true;
	while(mapName != "" && (bFirst || mapName != firstMap)) {
		
		bFirst = false;
		shortMapName = left(mapName, len(mapName) - 4);
		
		// Add to map list.
		if (len(maps[mapListIndex]) + len(shortMapName) > maxListEntrySize) {
			mapListIndex++;
			if (mapListIndex >= arrayCount(maps)) {
				control.nscLog("Insufficient space available for storing maps.");
				return;
			}
		}
		if (maps[mapListIndex] == "") {
			maps[mapListIndex] = shortMapName;
		} else {
			maps[mapListIndex] = maps[mapListIndex] $ separator $ shortMapName;
		}
		
		// Continue with next map.
		mapName = getMapName("", mapName, 1);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads a list of all mutators available on this server.
 *  $REQUIRE      !bInitialized
 *
 **************************************************************************************************/
function loadMutatorList() {
	local string mutator;
	local string description;
	local int index;
	local string mutatorInfoStr;
	local int mutatorListIndex;
	
	// For each mutator...
	getNextIntDesc("Mutator", 0, mutator, description);
	while (mutator != "") {
		
		// Get mutator info.
		if (instr(description, ",") >= 0) {
			description = left(description, instr(description, ","));
		}
		mutatorInfoStr = mutator $ separator $ description;
		
		// Store mutator info.
		if (mutatorListIndex >= arrayCount(mutators)) {
			control.nscLog("Insufficient space available for storing mutator info.");
			return;
		} else {
			mutators[mutatorListIndex] = mutatorInfoStr;
			mutatorListIndex++;
		}		
		
		// Continue with next mutator.
		getNextIntDesc("Mutator", ++index, mutator, description);	
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Performs a default installation of the Nexgen Server Controller. Calling this
 *                function will automatically configure the server for first use.
 *  $REQUIRE      bInitialized && !bInstalled
 *
 **************************************************************************************************/
function install() {
	local string packageName;
	
	packageName = class'NexgenUtil'.default.packageName;
	
	// Create unique key for this server.
	serverKey = class'NexgenUtil'.static.makeKey();
	
	// Add default game types.
	installDefaultGame("DM", "BotPack.DeathMatchPlus", "DM-");
	installDefaultGame("TDM", "BotPack.TeamGamePlus", "DM-");
	installDefaultGame("CTF", "BotPack.CTFGame", "CTF-");
	installDefaultGame("DOM", "BotPack.Domination", "DOM-");
	installDefaultGame("LMS", "BotPack.LastManStanding", "DM-");
	installDefaultGame("BR", "aBombingRun15.aBombingRun", "BR-");
	installDefaultGame("CTF4", "CTF4.CTF4Game", "CTF4-");
	
	// Add default account types.
	// 1 0 0 0 0 0 0 0 = Client may play on this server. If not set the client may only spectate.
	// 0 1 0 0 0 0 0 0 = Client has VIP access.
	// 0 0 1 0 0 0 0 0 = Admin slot access, i.e. may join if all player and vip slots are taken.
	// 0 0 0 1 0 0 0 0 = Don't check password for this player.
	// 0 0 0 0 1 0 0 0 = Player is a match admin, i.e. can influence the current match.
	// 0 0 0 0 0 1 0 0 = Moderator, can (un)ban, kick, mute, etc other players.
	// 0 0 0 0 0 0 1 0 = Is allowed to edit the game vote configurations.
	addAccountType("<Default>",     "10000000", "Player");    // No account.
	addAccountType("VIP",           "11000000");              // Registered players.
	addAccountType("Level 3 Admin", "11101000", "L3 Admin");
	addAccountType("Level 4 Admin", "11101100", "L4 Admin");
	addAccountType("Level 5 Admin", "11101110", "L5 Admin");
	addAccountType("Level 6 Admin", "11101110", "L6 Admin");
	addAccountType("Level 7 Admin", "11111110", "L7 Admin");
	
	// Set default settings.
	serverName = level.game.gameReplicationInfo.serverName;
	shortName = level.game.gameReplicationInfo.shortName;
	adminName = level.game.gameReplicationInfo.adminName;
	adminEmail = level.game.gameReplicationInfo.adminEmail;
	serverPassword = ""; // level.game.gamePassword; -- private :(
	spectatorsNeedPassword = false;
	playerSlots = level.game.maxPlayers;
	vipSlots = 2;
	adminSlots = 2;
	spectatorSlots = level.game.maxSpectators;
	autoUpdateBans = true;
	waitTime = 10;
	startTime = 5;
	scoreboardTime = 12;
	voteTime = 60;
	
	// Message HUD.
	useNexgenMessageHUD = true;
	replacementClass[0] = "BotPack.ChallengeHUD=" $ packageName $ ".NexgenHUDxDM";
	replacementClass[1] = "BotPack.ChallengeTeamHUD=" $ packageName $ ".NexgenHUDxTDM";
	replacementClass[2] = "BotPack.ChallengeCTFHUD=" $ packageName $ ".NexgenHUDxCTF";
	replacementClass[3] = "BotPack.AssaultHUD=" $ packageName $ ".NexgenHUDxAS";
	replacementClass[4] = "BotPack.ChallengeDominationHUD=" $ packageName $ ".NexgenHUDxDOM";
	
	// Server controller has been successfully installed.
	bInstalled = true;
	saveConfig();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Installs a default game type for game vote.
 *  $PARAM        gameName   Name of the new game type.
 *  $PARAM        gameClass  GameInfo class of the game type.
 *  $PARAM        mapPrefix  Default prefix used for maps that support this game type.
 *  $REQUIRE      bInitialized && !bInstalled
 *
 **************************************************************************************************/
function installDefaultGame(string gameName, string gameClass, string mapPrefix) {
	local int mapListIndex;
	local int listOffset;
	local int gameMapIndex;
	local string mapList;
	local string currentMap;
	local int index;
	local bool bFound;
	
	// Check if game class is available on the server.
	while (!bFound && index < arrayCount(games)) {
		if (instr(caps(games[index]), caps(gameClass) $ separator) >= 0) {
			bFound = true;
		} else {
			index++;
		}
	}
	// Cancel if game type isn't available.
	if (!bFound) {
		return;
	}
	
	// Find empty spot.
	bFound = false;
	index = 0;
	while (!bFound && index < arrayCount(gcGameClass)) {
		if (gcGameName[index] == "") {
			bFound = true;
		} else {
			index++;
		}
	}
	// Cancel if no space available.
	if (!bFound) {
		return;
	}
	
	gcGameName[index] = gameName;
	gcGameClass[index] = gameClass;
	
	listOffset = index * mapEntriesPerGame;
	
	// For each entry in the map list...
	while(mapListIndex < mapEntriesPerGame && maps[mapListIndex] != "") {
		mapList = maps[mapListIndex] $ separator;
		
		// Extract each map...
		while(instr(mapList, separator) > 0) {
			currentMap = left(mapList, instr(mapList, separator));
			mapList = mid(mapList, instr(mapList, separator) + 1);
			
			// And check if it should be added to the game config.
			if (len(currentMap) >= len(mapPrefix) &&
			    left(currentMap, len(mapPrefix)) ~= mapPrefix &&
			    instr(caps(currentMap), "TUTORIAL") < 0) {
				
				// Store map.
				if (len(gcGameMaps[listOffset + gameMapIndex]) + len(currentMap) > maxListEntrySize) {
					gameMapIndex++;
					if (gameMapIndex >= mapEntriesPerGame) {
						return; // Out of space.
					}
				}
				if (gcGameMaps[listOffset + gameMapIndex] == "") {
					gcGameMaps[listOffset + gameMapIndex] = currentMap;
				} else {
					gcGameMaps[listOffset + gameMapIndex] = gcGameMaps[listOffset + gameMapIndex] $
					                                        separator $ currentMap;
				}
			}
		}
		
		mapListIndex++;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds the specified account type to the server controller.
 *  $PARAM        typeName  Name of the account type to add.
 *  $PARAM        rights    The rights for this account type.
 *  $PARAM        title     Title for players that have this account name.
 *  $REQUIRE      typeName != ""
 *
 **************************************************************************************************/
function addAccountType(string typeName, string rights, optional string title) {
	local int index;
	local bool bFound;
	
	// Find an empty account type slot.
	while (!bFound && index < arrayCount(atTypeName)) {
		if (atTypeName[index] == "") {
			// This slot is empty.
			bFound = true;
			atTypeName[index] = typeName;
			atRights[index] = rights;
			atTitle[index] = title;
		} else {
			// This one is used, search for next one.
			index++;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Validates the current configuration. Any invalid settings will be automatically
 *                adjusted to a proper setting.
 *  $REQUIRE      bInitialized
 *  $RETURN       True, if the configuration was valid, false otherwise.
 *  $ENSURE       !old.checkConfig() ? new.checkConfig() : true
 *
 **************************************************************************************************/
function bool checkConfig() {
	local bool bConfigOk;
	local NexgenConfigChecker cCheck;

	cCheck = spawn(class'NexgenConfigChecker');
	bConfigOk = cCheck.checkConfig(self);
	cCheck.destroy();

	return bConfigOk;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Returns the index in the ban list for the given player info.
 *  $PARAM        playerName  Name of the player for which the entry in the ban list is to be found.
 *  $PARAM        playerIP    IP address of the player.
 *  $PARAM        playerID    ID code of the player.
 *  $RETURN       The index in the ban list for the specified player if banned, -1 if the player is
 *                not banned on the server.
 *  $ENSURE       0 <= result && result <= arrayCount(bannedName) || result == -1
 *
 **************************************************************************************************/
function int getBanIndex(string playerName, string playerIP, string playerID) {
	local int index;
	local bool bFound;
	local bool bNameMatch;
	local bool bIPMatch;
	local bool bIDMatch;
	
	// Lookup player in the ban list.
	while (!bFound && index < arrayCount(bannedName) && bannedName[index] != "") {
		
		bNameMatch = bannedName[index] ~= playerName;
		bIPMatch = instr(bannedIPs[index], playerIP) >= 0;
		bIDMatch = instr(bannedIDs[index], playerID) >= 0;
		
		// Match?
		if (bNameMatch || bIPMatch || bIDMatch) {
			// Oh yeah.
			bFound = true;
			
		} else {
			// Nope, maybe next.
			index++;
		}
		
	}
	
	// Return index in the ban list.
	if (bFound) {
		return index;
	} else {
		return -1;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Updates the specified ban entry. If a new IP or ID for the specified entry is
 *                detected it will be added.
 *  $PARAM        index     Location in the banlist.
 *  $PARAM        playerIP  IP address of the player.
 *  $PARAM        playerID  ID code of the player.
 *  $REQUIRE      0 <= index && index <= arrayCount(bannedName) && bannedName[index] != ""
 *  $ENSURE       instr(bannedIPs[index], playerIP) >= 0 && instr(bannedIDs[index], playerID) >= 0
 *
 **************************************************************************************************/
function updateBan(int index, string playerIP, string playerID) {
	local bool bIPMatch;
	local bool bIDMatch;
	
	bIPMatch = instr(bannedIPs[index], playerIP) >= 0;
	bIDMatch = instr(bannedIDs[index], playerID) >= 0;
	
	// Update ban if a new IP or ID is found for the player.
	if (!bIPMatch) {
		bannedIPs[index] = bannedIPs[index] $ separator $ playerIP;
	}
	if (!bIDMatch) {
		bannedIDs[index] = bannedIDs[index] $ separator $ playerID;
	}
	if (!bIPMatch || !bIDMatch) {
		saveConfig();
	}
}