/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenPanelListContainer
 *  $VERSION      1.00 (17-6-2008 18:12)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Nexgen control panel page container using a list combo.
 *
 **************************************************************************************************/
class NexgenPanelListContainer extends NexgenPanelContainer;

var UWindowComboControl panelSelector;            //

var NexgenPanel panels[32];                       // Panels displayed on this panel.
var int numPanels;                                // Number of panels displayed.
var int selectedPanel;                            //

const selectionPanelWidth = 192;                  //
const selectionPanelHeight = 19;                  //



/***************************************************************************************************
 *
 *  $DESCRIPTION  Creates the layout for this panel.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function created() {
	local float x, y, w, h;
	
	super(NexgenPanel).created();
	
	x = (winWidth - 4 - selectionPanelWidth) / 2;
	y = (selectionPanelHeight - defaultlistComboHeight) / 2;
	w = selectionPanelWidth;
	h = defaultlistComboHeight;
	
	panelSelector = UWindowComboControl(createWindow(class'UWindowComboControl', x, y, w, h));
	panelSelector.editBoxWidth = selectionPanelWidth;
	panelSelector.setEditable(false);
	
	selectedPanel = -1;
	
	panelSelector.register(self);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Prepares the window for the paint call.
 *  $PARAM        c  The canvas object which acts as a drawing surface for the dialog.
 *  $PARAM        x  Unknown.
 *  $PARAM        y  Unknown.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function beforePaint(Canvas c, float x, float y) {
	super.beforePaint(c, x, y);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Renders the GUI component.
 *  $PARAM        c  The canvas object which acts as a drawing surface for the dialog.
 *  $PARAM        x  Unknown.
 *  $PARAM        y  Unknown.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function paint(Canvas c, float x, float y) {
	local Region r;
	local Texture t;
	
	t = getLookAndFeelTexture();
	r = lookAndFeel.tabBackground;
	drawStretchedTextureSegment(c, 0, 0, winWidth, selectionPanelHeight, r.x, r.y, r.w, r.h, t);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the dialog of an event (caused by user interaction with the interface).
 *  $PARAM        control    The control object where the event was triggered.
 *  $PARAM        eventType  Identifier for the type of event that has occurred.
 *  $REQUIRE      control != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function notify(UWindowDialogControl control, byte eventType) {	
	if (control == panelSelector && eventType == DE_Change) {
		selectPage(panelSelector.getSelectedIndex());
	}
}

function selectPage(int index) {
	
	if (0 <= index && index < numPanels && index != selectedPanel) {
		if (0 <= selectedPanel && selectedPanel < numPanels) {
			panels[selectedPanel].hideWindow();
		}
		
		panels[index].showWindow();
		
		lookAndFeel.playMenuSound(self, MS_ChangeTab);
		
		selectedPanel = index;
	}	
}


/***************************************************************************************************
 *
 *  $DESCRIPTION  Retrieves the panel with the specified name.
 *  $PARAM        panelName  Name of the panel that is to be returned.
 *  $REQUIRE      panelName != ""
 *  $RETURN       The panel that was requested or none if the panel wasn't found.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function NexgenPanel getPanel(string panelName) {
	local NexgenPanel panel;
	local int index;
	
	// Search for panel.
	while (panel == none && index < numPanels) {
		if (panels[index].panelIdentifier ~= panelName) {
			panel = panels[index];
		} else if (panels[index].isA('NexgenPanelContainer')) {
			panel = NexgenPanelContainer(panels[index]).getPanel(panelName);
		}
		index++;
	}
	
	// Return result.
	return panel;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds a new NexgenPanel to the container or one of its subcontainers. To specify
 *                a specific parent use the parent parameter to indicate the path, e.g.
 *                "plugin,settings". If an invalid path is specified the panel won't be created.
 *  $PARAM        title       Text to display in the tab header.
 *  $PARAM        panelClass  Type of NexgenPanel to add/create.
 *  $PARAM        identifier  Identifier to assign to the new panel.
 *  $PARAM        parent      Path where to the parent of the new panel.
 *  $REQUIRE      panelClass != none
 *  $RETURN       The panel that was created and added to the container, or none if an invalid path
 *                to the parent container was specified.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function NexgenPanel addPanel(string title, class<NexgenPanel> panelClass,
                              optional string identifier, optional string parent) {
	local NexgenPanel newPanel;
	local string parentPanel;
	local string subPanels;
	local NexgenPanelContainer container;
	local bool bFound;
	local int index;
	local float desiredPanelHeight;
	
	// Add to subpanel?
	if (parent != "") {
		
		// Get local parent name.
		class'NexgenUtil'.static.split(parent, parentPanel, subPanels);
		
		// Locate local parent.
		while (!bFound && index < numPanels) {
			if (panels[index].isA('NexgenPanelContainer') &&
			    NexgenPanelContainer(panels[index]).panelIdentifier ~= parentPanel ) {
				bFound = true;
				newPanel = NexgenPanelContainer(panels[index]).addPanel(title, panelClass, identifier, subPanels);
			} else {
				index++;
			}
		}
		
	} else {
		
		// Nope, add it to the list panel container.
		if (numPanels < arrayCount(panels)) {
			
			// Create control.
			newPanel = NexgenPanel(createWindow(panelClass, 0, selectionPanelHeight, winWidth, winHeight - selectionPanelHeight));
			
			// Store in list.
			panels[numPanels] = newPanel;
			panelSelector.addItem(title, string(numPanels));
			
			// Set panel properties.
			if (identifier != "") {
				newPanel.panelIdentifier = identifier;
			}
			newPanel.client = self.client;
			newPanel.setContent();
			
			// Make sure the right panel is selected.
			if (numPanels == 0) {
				selectedPanel = 0;
				panelSelector.setSelectedIndex(0);
			} else {
				newPanel.hideWindow();
			}
			
			numPanels++;
		}
		
	}
	
	// Return created panel.
	return newPanel;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Selects the panel with the specified name.
 *  $PARAM        panelName  The name of the panel that is to be selected.
 *  $RETURN       True if the panel was selected, false if it wasn't found.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool selectPanel(string panelName) {
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the sub panels on this panel that the server configuration has been
 *                updated.
 *  $PARAM        configType  Type of settings that have been changed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function configChanged(byte configType) {
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies this panel that the extended game info has been updated.
 *  $PARAM        infoType  Type of information that has been changed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function gameInfoChanged(byte infoType) {
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the client of a player event. Additional arguments to the event should be
 *                combined into one string which then can be send along with the playerEvent call.
 *  $PARAM        playerNum  Player identification number.
 *  $PARAM        eventType  Type of event that has occurred.
 *  $PARAM        args       Optional arguments.
 *  $REQUIRE      playerNum >= 0
 *  $OVERRIDE
 *
 **************************************************************************************************/
function playerEvent(int playerNum, name eventType, optional string args) {
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a general event has occurred in the system.
 *  $PARAM        type      The type of event that has occurred.
 *  $PARAM        argument  Optional arguments providing details about the event.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function notifyEvent(string type, optional string arguments) {
}
