/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenHUDWrapper
 *  $VERSION      1.02 (6-4-2008 11:46)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  HUD Wrapper class. Used to support the Nexgen message HUD with custom HUDs. This
 *                class simply passes all calls to the original HUD, except in the cases where the
 *                Nexgen message HUD kicks in.
 *
 **************************************************************************************************/
class NexgenHUDWrapper extends ChallengeHUD;

var ChallengeHUD originalHUD;           // Original HUD.
var NexgenHUD nscHUD;                   // The Nexgen HUD instance (which will do the rendering).



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the HUD wrapper.
 *  $PARAM        originalHUD  The original HUD that was used by the player.
 *  $PARAM        nscHUD       The Nexgen message HUD.
 *  $REQUIRE      originalHUD != none && nscHUD != none
 *
 **************************************************************************************************/
function initialize(ChallengeHUD originalHUD, NexgenHUD nscHUD) {
	local int index;
	
	// Copy control variables.
	self.originalHUD = originalHUD;
	self.nscHUD = nscHUD;
	playerOwner = originalHUD.playerOwner;
	
	// Clear messages.
	for (index = 0; index < arrayCount(shortMessageQueue); index++) {
		originalHUD.message(none, "", '');
	}
	for (index = 0; index < arrayCount(localMessages); index++) {
		originalHUD.localizedMessage(class'NexgenNullMessage');
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Prepares the system for the HUD render phase. Probably never used.
 *  $PARAM        c  The canvas object on which the rendering will be done.
 *  $REQUIRE      c != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated event preRender(Canvas c) {
	if (originalHUD != none) originalHUD.preRender(c);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Renders the HUD. It will simply call the postRender() function of the original HUD
 *                and also calls the renderMessageBox() function on the NexgenHUD.
 *  $PARAM        c  The canvas object on which the rendering will be done.
 *  $REQUIRE      c != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated event postRender(Canvas c) {
	local Console cons;
	local bool bWasTyping;
	
	nscHUD.preRenderHUD(c);
	
	if (originalHUD != none) {
		// Copy HUD settings.
		originalHUD.bHideHUD         = bHideHUD;
		originalHUD.bHideAllWeapons  = bHideAllWeapons;
		originalHUD.bHideStatus      = bHideStatus;
		originalHUD.bHideAmmo        = bHideAmmo;
		originalHUD.bHideTeamInfo    = bHideTeamInfo;
		originalHUD.bHideFrags       = bHideFrags;
		originalHUD.bUseTeamColor    = bUseTeamColor;
		originalHUD.opacity          = opacity;
		originalHUD.hudScale         = hudScale;
		originalHUD.weaponScale      = weaponScale;
		originalHUD.statusScale      = statusScale;
		originalHUD.favoriteHUDColor = favoriteHUDColor;
		originalHUD.crosshairColor   = CrosshairColor;
		originalHUD.hudMutator       = hudMutator;
		
		// Save data & fool the original HUD.
		cons = playerOwner.player.console;
		bWasTyping = cons.bTyping;
		cons.bTyping = false;
		
		originalHUD.bHideFaces = true;
		
		// Let the original HUD render it's stuff.
		originalHUD.postRender(c);
		
		// Restore data.
		cons.bTyping = bWasTyping;
	}
	
	nscHUD.postRenderHUD(c);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the player switches to the specified weapon group. Doesn't appear to
 *                be used anywhere.
 *  $PARAM        f  The number of the weapon group to which the player is about to switch.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function inputNumber(byte f) {
	if (originalHUD != none) originalHUD.inputNumber(f);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Changes the HUD mode. Not used in Unreal Tournament.
 *  $PARAM        d  Number to add to the HUD mode number (delta).
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function changeHud(int d) {
	if (originalHUD != none) originalHUD.changeHud(d);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Changes the crosshair to use.
 *  $PARAM        d  Number to add to the crosshair number (delta).
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function changeCrosshair(int d) {
	if (originalHUD != none) originalHUD.changeCrosshair(d);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Renders the crosshair on the screen.
 *  $PARAM        c       The canvas object on which the rendering will be done.
 *  $PARAM        startX  Horizontal offset.
 *  $PARAM        startX  Vertical offset.
 *  $REQUIRE      c != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function drawCrossHair(Canvas c, int startX, int startY) {
	if (originalHUD != none) originalHUD.drawCrossHair(c, startX, startY);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Displays a new message on the screen.
 *  $PARAM        pri      Information about the player that is related with this message.
 *  $PARAM        msg      String containing the message to be displayed.
 *  $PARAM        msgType  Type of the message.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function message(PlayerReplicationInfo pri, coerce string msg, name msgType) {
	local bool bHandleByParent;
	
	// Check class responsible for this message.
	bHandleByParent = msgType == 'CriticalEvent';
	
	// Handle message.
	if (bHandleByParent) {
		if (originalHUD != none) originalHUD.message(pri, msg, msgType);
	} else {
		addMessage(msg, msgType, pri, none);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Displays a new localized message on the screen.
 *  $PARAM        message         Class used to construct the localized message string.
 *  $PARAM        switch          Message selection switch number.
 *  $PARAM        relatedPRI_1    First player that is related to this message.
 *  $PARAM        relatedPRI_2    Second player that is related to this message.
 *  $PARAM        optionalObject  Object involved in the construction of the message string.
 *  $PARAM        criticalString  Critical message string.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function localizedMessage(class<LocalMessage> message,
                                    optional int switch,
                                    optional PlayerReplicationInfo relatedPRI_1,
                                    optional PlayerReplicationInfo relatedPRI_2,
                                    optional Object optionalObject,
                                    optional string criticalString) {
	local bool bHandleByParent;
	local string msgStr;
	
	// Check class responsible for this message.
	bHandleByParent = message.default.bIsSpecial;
	
	// Handle message.
	if (bHandleByParent) {
		if (originalHUD != none) originalHUD.localizedMessage(message, switch, relatedPRI_1, relatedPRI_2, optionalObject, criticalString);
	} else {
		// Get message string.
        if (message.default.bComplexString) {
            msgStr = criticalString;
        } else {
            msgStr = message.static.getString(switch, relatedPRI_1, relatedPRI_2, optionalObject);
        }
		
		// Add the message.
		addMessage(msgStr, '', relatedPRI_1, relatedPRI_2);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Displays the specified message. Doesn't appear to be used.
 *  $PARAM        s      The message to display.
 *  $PARAM        pName  Name of the player that created the message?
 *  $PARAM        pZone  Location of the player that created the message?
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function playReceivedMessage(string s, string pName, ZoneInfo pZone) {
	if (originalHUD != none) originalHUD.playReceivedMessage(s, pName, pZone);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether this HUD is responsible for displaying messages.
 *  $RETURN       True if this HUD handles the messages instead of the console.
 *  $ENSURE       result == true
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function bool displayMessages(Canvas c) {
    return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called by the engine when a key, mouse, or joystick button is pressed or released,
 *                or any analog axis movement is processed.
 *  $PARAM        key     The key that was involved.
 *  $PARAM        action  The action that was taken on the involved key.
 *  $PARAM        delta   Amount of time elapsed/used for the action.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool processKeyEvent(int key, int action, float delta) {
	if (originalHUD != none) return originalHUD.processKeyEvent(key, action, delta);
    return false;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Sets the location where the player took damage.
 *  $PARAM        hitLoc  The location where the player was hit.
 *  $PARAM        damage  The amount of damage taken by the player.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function setDamage(vector hitLoc, float damage) {
	if (originalHUD != none) originalHUD.setDamage(hitLoc, damage);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Draws the typing prompt on the specified canvas.
 *  $PARAM        canv  The canvas on which the typing prompt should be rendered.
 *  $PARAM        cons  Console containing the typed message.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function drawTypingPrompt(Canvas canv, Console cons) {
	// Suppress! This is taken care of in NexgenHUD.renderMessageBox().
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds a new message to display. If the message can't be added directly because the
 *                NexgenHUD object hasn't been set, the message will be buffered.
 *  $PARAM        msg      String containing the message to add.
 *  $PARAM        msgType  Type of the message.
 *  $PARAM        pri1     First player that is related to this message.
 *  $PARAM        pri2     Second player that is related to this message.
 *
 **************************************************************************************************/
simulated function addMessage(string msg, name msgType, PlayerReplicationInfo pri1, PlayerReplicationInfo pri2) {
	if (nscHUD != none) nscHUD.message(msg, msgType, pri1, pri2);
}

