/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenHUDxTDM
 *  $VERSION      1.01 (31-12-2006 16:01)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Extended TDM HUD that uses the Nexgen HUD to render the messages.
 *
 **************************************************************************************************/
class NexgenHUDxTDM extends ChallengeTeamHUD;

struct SavedMessage {                   // Structure used for buffered messages.
	var string msg;                     // Message string.
	var name msgType;                   // Type of the message.
	var PlayerReplicationInfo pri1;     // Related player information 1.
	var PlayerReplicationInfo pri2;     // Related player information 2.
};

var NexgenHUD nscHUD;                   // The Nexgen HUD instance (which will do the rendering).
var float lastCheck;                    // Last time we checked if the NexgenHUD is available.
var SavedMessage savedMessages[10];     // Messages received before the NexgenHUD was available.
var int savedMsgCount;                  // Number of messages saved.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Renders the HUD. It will simply call the postRender() function of the parent class
 *                and the renderMessageBox() function on the NexgenHUD if it is available. When the
 *                NexgenHUD is not available an attempt is made to locate it.
 *  $PARAM        c  The canvas object on which the rendering will be done.
 *  $REQUIRE      c != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function postRender(Canvas c) {
	if (nscHUD == none) {
		setNexgenHUD();
		super.postRender(c);
	} else {
		nscHUD.preRenderHUD(c);
		super.postRender(c);
		nscHUD.postRenderHUD(c);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Displays a new message on the screen.
 *  $PARAM        pri      Information about the player that is related with this message.
 *  $PARAM        msg      String containing the message to be displayed.
 *  $PARAM        msgType  Type of the message.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function message(PlayerReplicationInfo pri, coerce string msg, name msgType) {
	local bool bHandleByParent;
	
	// Check class responsible for this message.
	bHandleByParent = msgType == 'CriticalEvent';
	
	// Handle message.
	if (bHandleByParent) {
		super.message(pri, msg, msgType);
	} else {
		addMessage(msg, msgType, pri, none);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Displays a new localized message on the screen.
 *  $PARAM        message         Class used to construct the localized message string.
 *  $PARAM        switch          Message selection switch number.
 *  $PARAM        relatedPRI_1    First player that is related to this message.
 *  $PARAM        relatedPRI_2    Second player that is related to this message.
 *  $PARAM        optionalObject  Object involved in the construction of the message string.
 *  $PARAM        criticalString  Critical message string.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function localizedMessage(class<LocalMessage> message,
                                    optional int switch,
                                    optional PlayerReplicationInfo relatedPRI_1,
                                    optional PlayerReplicationInfo relatedPRI_2,
                                    optional Object optionalObject,
                                    optional string criticalString) {
	local bool bHandleByParent;
	local string msgStr;
	
	// Check class responsible for this message.
	bHandleByParent = message.default.bIsSpecial;
	
	// Handle message.
	if (bHandleByParent) {
		super.localizedMessage(message, switch, relatedPRI_1, relatedPRI_2, optionalObject, criticalString);
	} else {
		// Get message string.
        if (message.default.bComplexString) {
            msgStr = criticalString;
        } else {
            msgStr = message.static.getString(switch, relatedPRI_1, relatedPRI_2, optionalObject);
        }
		
		// Add the message.
		addMessage(msgStr, '', relatedPRI_1, relatedPRI_2);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Draws the typing prompt on the specified canvas.
 *  $PARAM        canv  The canvas on which the typing prompt should be rendered.
 *  $PARAM        cons  Console containing the typed message.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function drawTypingPrompt(Canvas canv, Console cons) {
	// Suppress! This is taken care of in NexgenHUD.renderMessageBox().
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Attemps to locate the NexgenHUD instance. Once the HUD has been located any saved
 *                messages will be restored. To save processing resources the attemps will have a
 *                minimum interval of 0.2 seconds.
 *  $REQUIRE      nscHUD == none
 *  $ENSURE       nscHUD != none ? savedMsgCount == 0 : true
 *
 **************************************************************************************************/
simulated function setNexgenHUD() {
	local int index;
	
	if (level.timeSeconds - lastCheck >= 0.2) {
		// Locate NexgenHUD instance.
		lastCheck = level.timeSeconds;
		foreach allActors(class'NexgenHUD', nscHUD) {
			if (nscHUD != none) {
				break;
			}
		}
		
		// Restore saved messages.
		if (nscHUD != none) {
			for (index = 0; index < savedMsgCount; index++) {
				nscHUD.message(savedMessages[index].msg, savedMessages[index].msgType,
				               savedMessages[index].pri1, savedMessages[index].pri2);
			}
			savedMsgCount = 0;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds a new message to display. If the message can't be added directly because the
 *                NexgenHUD object hasn't been set, the message will be buffered.
 *  $PARAM        msg      String containing the message to add.
 *  $PARAM        msgType  Type of the message.
 *  $PARAM        pri1     First player that is related to this message.
 *  $PARAM        pri2     Second player that is related to this message.
 *
 **************************************************************************************************/
simulated function addMessage(string msg, name msgType, PlayerReplicationInfo pri1, PlayerReplicationInfo pri2) {
	local int index;
	
	if (nscHUD == none) {
		// Buffer the message.
		if (savedMsgCount < arrayCount(savedMessages)) {
			index = savedMsgCount;
			savedMsgCount++;
		} else {
			for (index = 1; index < savedMsgCount; index++) {
				savedMessages[index - 1] = savedMessages[index];
			}
			savedMsgCount = savedMsgCount - 1;
		}
		savedMessages[index].msg = msg;
		savedMessages[index].msgType = msgType;
		savedMessages[index].pri1 = pri1;
		savedMessages[index].pri2 = pri2;
	} else {
		// Pass it to the Nexgen HUD.
		nscHUD.message(msg, msgType, pri1, pri2);
	}
}