/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenRCPReplacementHUDSettings
 *  $VERSION      1.03 (20-6-2008 14:14)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Nexgen about control panel page.
 *
 **************************************************************************************************/
class NexgenRCPReplacementHUDSettings extends NexgenPanel;

var NexgenClientCore rpci;                        // Remote Procedure Call interface.

var NexgenSimpleListBox hudReplacementList;
var UWindowSmallButton hudReplSaveButton;
var UWindowSmallButton hudReplRemButton;
var NexgenEditControl originalHUDClassInp;
var NexgenEditControl replacementHUDClassInp;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Creates the contents of the panel.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function setContent() {	
	
	// Create layout & add components.
	createPanelRootRegion();
	splitRegionV(256, defaultComponentDist);
	hudReplacementList = NexgenSimpleListBox(addListBox(class'NexgenSimpleListBox'));
	divideRegionH(4);
	addLabel(client.lng.hudReplaceClassesTxt, true, TA_Center);
	splitRegionV(128);
	splitRegionV(128);
	splitRegionV(192);
	addLabel(client.lng.originalHUDClassTxt);
	originalHUDClassInp = addEditBox();
	addLabel(client.lng.replacementHUDClassTxt);
	replacementHUDClassInp = addEditBox();
	divideRegionV(2, defaultComponentDist);
	skipRegion();
	hudReplSaveButton = addButton(client.lng.saveTxt);
	hudReplRemButton = addButton(client.lng.removeTxt);
	
	// Configure components.
	originalHUDClassInp.setMaxLength(64);
	replacementHUDClassInp.setMaxLength(64);
	loadHUDReplacementList();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads the HUD class replacement list.
 *
 **************************************************************************************************/
function loadHUDReplacementList() {
	local int index;
	local NexgenSimpleListItem item;
	
	// Clear list.
	hudReplacementList.selectedItem = none;
	hudReplacementList.items.clear();
	
	// Add weapon classes.
	while (index < arrayCount(client.sConf.replacementClass) &&
	       client.sConf.replacementClass[index] != "") {
		
		item = NexgenSimpleListItem(hudReplacementList.items.append(class'NexgenSimpleListItem'));
		item.displayText = client.sConf.replacementClass[index];
		item.itemID = index;
		
		index++;
	}
	
	// Add 'add new item' option if list isn't full.
	if (index < arrayCount(client.sConf.replacementClass)) {
		item = NexgenSimpleListItem(hudReplacementList.items.insert(class'NexgenSimpleListItem'));
		item.displayText = client.lng.addNewItemTxt;
		item.itemID = -1;
	}
	
	// Signal item select event.
	hudReplacementSelected();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when an item from the hud replacement list was selected.
 *
 **************************************************************************************************/
function hudReplacementSelected() {
	local NexgenSimpleListItem item;
	local string str;

	// Get selected item.
	item = NexgenSimpleListItem(hudReplacementList.selectedItem);
	
	// Update GUI.
	hudReplSaveButton.bDisabled = (item == none);
	hudReplRemButton.bDisabled = (item == none) || (item.itemID < 0);
	originalHUDClassInp.setDisabled((item == none));
	replacementHUDClassInp.setDisabled((item == none));
	
	if (item == none) {
		originalHUDClassInp.setValue("");
		replacementHUDClassInp.setValue("");
	} else if (item.itemID < 0) {
		originalHUDClassInp.setValue("");
		replacementHUDClassInp.setValue("");
	} else {
		str = client.sConf.replacementClass[item.itemID];
		originalHUDClassInp.setValue(left(str, instr(str, "=")));
		replacementHUDClassInp.setValue(mid(str, instr(str, "=") + 1));
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Attemps to locate the RPC interface for this control panel.
 *  $REQUIRE      client != none
 *  $RETURN       True if the RPC interface has been set, false if not.
 *  $ENSURE       result == true ? rcpi != none : true
 *
 **************************************************************************************************/
function bool setRPCI() {
	
	// Check if RPC interface is already set.
	if (rpci == none) {
		// Attempt to get the RPC interface.
		rpci = NexgenClientCore(client.getController(class'NexgenClientCore'.default.ctrlID));
		return rpci != none;
		
	} else {
		// It is.
		return true;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the dialog of an event (caused by user interaction with the interface).
 *  $PARAM        control    The control object where the event was triggered.
 *  $PARAM        eventType  Identifier for the type of event that has occurred.
 *  $REQUIRE      control != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function notify(UWindowDialogControl control, byte eventType) {	
	super.notify(control, eventType);
	
	// HUD replacement selected?
	if (control == hudReplacementList && eventType == DE_Click) {
		hudReplacementSelected();
	}
	
	// Remove HUD replacement selected?
	if (control == hudReplRemButton && eventType == DE_Click && !hudReplRemButton.bDisabled && setRPCI()) {
		rpci.delHUDReplacementClass(NexgenSimpleListItem(hudReplacementList.selectedItem).itemID);
	}
	
	// Save HUD replacement selected?
	if (control == hudReplSaveButton && eventType == DE_Click && !hudReplSaveButton.bDisabled && setRPCI()) {
		rpci.saveHUDReplacementClass(NexgenSimpleListItem(hudReplacementList.selectedItem).itemID,
		                             class'NexgenUtil'.static.trim(originalHUDClassInp.getValue()),
		                             class'NexgenUtil'.static.trim(replacementHUDClassInp.getValue()));
	}
	
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies this panel that the server configuration has been updated.
 *  $PARAM        configType  Type of settings that have been changed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function configChanged(byte configType) {
	
	// Relevant settings for this panel?
	if (configType == client.sConf.CT_HUDReplacementList) {
		loadHUDReplacementList();
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	panelIdentifier="replacementhudsettings"
	panelHeight=90
}
