/***************************************************************************************************
 *
 *  NGLS. Nexgen Global Login System by Zeropoint.
 *
 *  $CLASS        NGLSHTTPClient
 *  $VERSION      1.03 (15-11-2008 14:50)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  The HTTP client used to verify login information.
 *
 **************************************************************************************************/
class NGLSHTTPClient extends UBrowserHTTPClient;

var NGLSMain xControl;                  // Global login system controller.

var NGLSClient pendingClientChecks[32]; // Clients that still have to be checked.
var bool bClientCheckInProgress;        // Current checking login information of a client?

// Commands.
const CMD_AcceptLogin = "ACCEPT";       // Accepts the login request of a player.
const CMD_RejectLogin = "REJECT";       // Rejects the login request of a player.

// Command arguments.
const CMD_AcceptLogin_GameID = 0;
const CMD_AcceptLogin_PlayerNum = 1;

const CMD_RejectLogin_GameID = 0;
const CMD_RejectLogin_PlayerNum = 1;
const CMD_RejectLogin_Reason = 2;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the UTStats statistics retriever client.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function preBeginPlay() {
	super.preBeginPlay();
	
	// Get controller.
	xControl = NGLSMain(owner);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether the server settings are configured properly so 
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool serverSettingsOk() {
	return (class'NexgenUtil'.static.trim(xControl.xConf.nglsServerPath) != "") &&
	       (class'NexgenUtil'.static.trim(xControl.xConf.nglsServerHost) != "");
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Sends a HTTP get request to the server to check the login information for the
 *                specified client.
 *  $PARAM        xClient  The client for which the login information is to be checked.
 *  $REQUIRE      xClient != none
 *
 **************************************************************************************************/
function verifyLoginInfo(NGLSClient xClient) {
	local string url;
	
	if (!serverSettingsOk()) return;
	
	enqueueClient(xClient);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds the specified client to the list of clients to check.
 *  $PARAM        xClient  The client for which the login information is to be checked.
 *  $REQUIRE      xClient != none
 *
 **************************************************************************************************/
function enqueueClient(NGLSClient xClient) {
	local int index;
	local bool bFound;
	
	// Compact queue (for clients that left the server).
	compactQueue();

	// Add client to the queue.
	while (!bFound && index < arrayCount(pendingClientChecks)) {
		if (pendingClientChecks[index] == xClient) {
			// Client is already in the queue.
			bFound = true;
		} else if (pendingClientChecks[index] == none) {
			// Empty slot found, store client in the queue.
			pendingClientChecks[index] = xClient;
			bFound = true;
		} else {
			// Slot already used, check next slot.
			index++;
		}
	}
	
	// Check next client in the queue.
	doNextClientCheck();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Compacts the queue of clients whose login information is to be checked.
 *
 **************************************************************************************************/
function compactQueue() {
	local int index;
	local int lastEmpty;
	
	lastEmpty = -1;
	for (index = 0; index < arrayCount(pendingClientChecks); index++) {
		if (lastEmpty < 0 && pendingClientChecks[index] == none) {
			lastEmpty = index;
		} else if (lastEmpty > 0 && pendingClientChecks[index] != none) {
			pendingClientChecks[lastEmpty++] = pendingClientChecks[index];
			pendingClientChecks[index] = none;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks the login information for the next client in the queue.
 *
 **************************************************************************************************/
function doNextClientCheck() {
	local NGLSClient xClient;
	local string url;
	
	if (!bClientCheckInProgress && pendingClientChecks[0] != none) {
		
		bClientCheckInProgress = true;
		xClient = pendingClientChecks[0];
		
		xControl.nglsLog(class'NexgenUtil'.static.format(xControl.lng.startCheckMsg, xClient.client.playerName));

		// Construct request url.
		url = xControl.xConf.nglsServerPath $
		      "?checkplayer=" $ xClient.client.playerNum $
		      "&gameid="      $ xControl.gameID $
		      "&username="    $ class'NexgenUtil'.static.urlEncode(xClient.loginUsername) $
		      "&password="    $ xClient.loginPasswordHash;
	
		// Send request.	
		browse(xControl.xConf.nglsServerHost, url, xControl.xConf.nglsServerPort);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Removes the specified client from the queue.
 *  $PARAM        xClient  The client that is to be removed from the server.
 *  $REQUIRE      xClient != none
 *
 **************************************************************************************************/
function removeClientFromQueue(NGLSClient xClient) {
	local int index;
	
	for (index = 0; index < arrayCount(pendingClientChecks); index++) {
		if (pendingClientChecks[index] == xClient) {
			pendingClientChecks[index] = none;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the connection has closed.
 *
 **************************************************************************************************/
event closed() {
	super.closed();
	
	// Check next client in the queue.
	if (bClientCheckInProgress) {
		pendingClientChecks[0] = none;
		bClientCheckInProgress = false;
	}
	compactQueue();
	doNextClientCheck();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the HTTP request failed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function HTTPError(int code) {
	xControl.nglsLog(class'NexgenUtil'.static.format(xControl.lng.nglsHTTPErrorMsg, code));
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the HTTP request has been replied and the data has been received.
 *  $PARAM        data  The data that has been received.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function HTTPReceivedData(string data) {
	local string remaining;
	local string currLine;
	
	// Process data.
	remaining = data;
	do {
		currLine = class'NexgenUtil'.static.trim(class'NexgenUtil'.static.getNextLine(remaining));
		if (currLine != "") {
			processData(currLine);
		}
	} until (remaining == "");
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Splits the given string in two parts: the first line and the rest.
 *  $PARAM        str  The string that should be splitted.
 *  $RETURN       The first line in the given string.
 *
 **************************************************************************************************/
function processData(string str) {
	local string cmdType;
	local string cmdArgs[10];
	local NGLSClient xClient;
	
	// Parse command.
	if (class'NexgenUtil'.static.parseCommandStr("nsc" @ str, cmdType, cmdArgs)) {
		switch (cmdType) {
			case CMD_AcceptLogin:
				if (cmdArgs[CMD_AcceptLogin_GameID] == xControl.gameID) {
					xClient = xControl.getClientByNum(int(cmdArgs[CMD_AcceptLogin_PlayerNum]));
					if (xClient != none) {
						xClient.acceptLogin();
						removeClientFromQueue(xClient);
					}
				}
				break;
				
			case CMD_RejectLogin:
				if (cmdArgs[CMD_RejectLogin_GameID] == xControl.gameID) {
					xClient = xControl.getClientByNum(int(cmdArgs[CMD_RejectLogin_PlayerNum]));
					if (xClient != none) {
						xClient.rejectLogin(cmdArgs[CMD_RejectLogin_Reason]);
						removeClientFromQueue(xClient);
					}
				}
				break;
		}
	}
}