/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenXClient
 *  $VERSION      1.06 (30-11-2007 19:32)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Extension pack client controller class. This class is the base of the clientside
 *                support for the extension plugin.
 *
 **************************************************************************************************/
class NexgenXClient extends NexgenClientController;

var NexgenX xControl;                   // Extension controller
var NexgenXLang lng;                    // Language instance to support localization.

var NexgenXRCPMapSwitch mapSwitchTab;   // Map switch control panel tab.

var NexgenXPlayerOverlay playerOverlay; // Player overlay skin.

// Map list control.
var bool bSendingMapList;               // Whether the server is currently sending the map list.
var bool bMapListSend;                  // Whether the map list has been sended to the client.
var string firstMap;                    // Filename of the first map on the server.
var string lastMap;                     // Filename of the last map send to the client.
var string maps[1024];                  // Maps available on the server.
var int mapCount;                       // Number of maps available.

// Dynamic control info.
var int lastCountDown;                  // Last known value of gInf.countDown.

// Controller settings.
const timerFreq = 5.0;                  // Timer tick frequency.

// General constants.
const maxStringSize = 255;              // Maximum size of strings replicated over the net.
const separator = ",";                  // Character used to seperate elements in a list.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {
	reliable if (role == ROLE_Authority) // Replicate to client...
		receiveMapListPart;
		
	reliable if (role == ROLE_SimulatedProxy) // Replicate to server...
		requestMapList, doMapSwitch;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Time critical event detection loop.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function tick(float deltaTime) {
	
	// Client side actions.
	if (role == ROLE_SimulatedProxy && client != none && !client.bNetWait) {
		
		// Game starting count down ticked?
		if (client.gInf.gameState == client.gInf.GS_Starting && lastCountDown != client.gInf.countDown) {
			lastCountDown = client.gInf.countDown;
			
			// Play countdown announcer sound?
			if (1 <= lastCountDown && lastCountDown <= 10 && client.player.myHUD.isA('ChallengeHUD')) {
				ChallengeHUD(client.player.myHUD).tellTime(16 - lastCountDown);
			}
		}
		
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the client controller. This function is automatically called after
 *                the critical variables have been set, such as the client variable.
 *  $PARAM        creator  The Actor that has added the controller to the client.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function initialize(optional Actor creator) {
	xControl = NexgenX(creator);
	lng = xControl.lng;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Modifies the setup of the Nexgen remote control panel.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function setupControlPanel() {
	
	// Load localization support.
	lng = spawn(class'NexgenXLang');

	// Match.
	if (client.hasRight(client.R_MatchAdmin)) {
		requestMapList();
		mapSwitchTab = NexgenXRCPMapSwitch(client.mainWindow.mainPanel.addPanel(lng.mapSwitchTabTxt, class'NexgenXRCPMapSwitch', , "game"));
	}

}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called by the client when it wants to receive the list of maps that are available
 *                on the server.
 *  $ENSURE       bSendingMapList || bMapListSend
 *
 **************************************************************************************************/
function requestMapList() {
	
	// Check if map list is already being send or has already been send.
	if (bSendingMapList || bMapListSend) {
		// It is and there's no need to send things again.
		return;
	}
	
	// Start sending the map list to the client.
	bSendingMapList = true;
	setTimer(1.0 / timerFreq, true);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Timer tick function. On the server side the timer tick is responsible for sending
 *                the map list to the client if bSendingMapList is set to true.
 *
 **************************************************************************************************/
function timer() {
	local string mapList;
	local string nextMap;
	local bool bMapListComplete;
	local bool bMapListFull;
	
	// Send next part of the map list.
	if (bSendingMapList) {
		
		// Just starting to send the map list?
		if (firstMap == "") {
			// First map hasn't been determined yet, so yes.
			firstMap = getMapName("", "", 0);
			mapList = firstMap;
			lastMap = firstMap;
		}
		
		// Get next map to send.
		nextMap = getMapName("", lastMap, 1);
		
		// Continue adding maps the map list till the string get too large.
		while (!bMapListComplete && !bMapListFull) {
			
			if (nextMap ~= firstMap) {
				// Back at the first map, so we're done.
				bMapListComplete = true;
			} else if (mapList == "")  {
				// Map list is empty, so we can safely add the next map.
				mapList = nextMap;
				lastMap = nextMap;
				nextMap = getMapName("", lastMap, 1);
			} else if (len(mapList) + len(separator) + len(nextMap) > maxStringSize) {
				// Can't add the next map without exceeding the maximum string size.
				bMapListFull = true;
			} else {
				// There's still enough space to add the next map.
				mapList = mapList $ separator $ nextMap;
				lastMap = nextMap;
				nextMap = getMapName("", lastMap, 1);
			}
		}
		
		// Send the map list.
		receiveMapListPart(bMapListComplete, mapList);
		
		// Check if sending has been completed.
		if (bMapListComplete) {
			bSendingMapList = false;
			bMapListSend = true;
			setTimer(0.0, false);
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called on the client when a new part of the map list has been received.
 *  $PARAM        bLastPart    Indicates whether this is the last part of the map list.
 *  $PARAM        mapListPart  String containing a part of the map list.
 *
 **************************************************************************************************/
simulated function receiveMapListPart(bool bLastPart, string mapListPart) {
	local string remaining;
	local string mapName;
	
	remaining = mapListPart;
	while (remaining != "" && mapCount < arrayCount(maps)) {
		class'NexgenUtil'.static.split(remaining, mapName, remaining);
		maps[mapCount++] = mapName;
	}
	
	if (bLastPart) {
		mapSwitchTab.notifyMapListAvailable();	
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Causes the server to load the specified map with the given game type and mutators.
 *  $PARAM        mapName   File name of the map that is to be loaded.
 *  $PARAM        gameType  The game type to use when the new map is loaded.
 *  $PARAM        mutators  List of mutators that are to be loaded.
 *
 **************************************************************************************************/
function doMapSwitch(string mapName, byte gameType, string mutators) {
	local string remaining;
	local string indexStr;
	local int index;
	local string mutatorClass;
	local string mutatorClasses;
	local string temp;
	local string gameClass;
	local string bootURL;
	local string mapNameNoExt;
	
	// Preliminary checks.
	if (!client.hasRight(client.R_MatchAdmin)) {
		return;
	}
	
	// Get game type.
	if (0 <= gameType && gameType < arrayCount(client.sConf.gameTypeInfo)) {
		class'NexgenUtil'.static.split(client.sConf.gameTypeInfo[gameType], gameClass, temp);
	} else {
		gameClass = "";
	}
	
	// Get mutator classes.
	remaining = mutators;
	while (remaining != "") {
		class'NexgenUtil'.static.split(remaining, indexStr, remaining);
		index = int(indexStr);
		if (0 <= index && index < arrayCount(client.sConf.mutatorInfo)) {
			class'NexgenUtil'.static.split(client.sConf.mutatorInfo[index], mutatorClass, temp);
			if (mutatorClasses == "") {
				mutatorClasses = mutatorClass;
			} else {
				mutatorClasses = mutatorClasses $ separator $ mutatorClass;
			}
		}
	}
	
	// Assemble URL.
	bootURL = mapName $ "?game=" $ gameClass $ "?mutator=" $ mutatorClasses;
	level.serverTravel(bootURL, false);
	
	// Announce event.
	mapNameNoExt = left(mapName, instr(mapName, "."));
	control.broadcastMsg(lng.adminMapSwitchMsg, client.playerName, mapNameNoExt, , , client.player.playerReplicationInfo);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds or removes the player overlay skin for this client.
 *  $PARAM        bRemove  Whether the overlay skin is to be removed.
 *  $ENSURE       bRemove ? new.playerOverlay == none : new.playerOverlay != none
 *
 **************************************************************************************************/	
function setPlayerOverlay(optional bool bRemove) {
	
	// Remove overlay skin?
	if (bRemove && playerOverlay != none) {
		// Yes.
		playerOverlay.destroy();
		playerOverlay = none;
	
	} else if (!bRemove && playerOverlay == none){
		// No, add overlay skin.
		playerOverlay = spawn(class'NexgenXPlayerOverlay', client.player, ,
		                      client.player.location, client.player.rotation);
		playerOverlay.client = client;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	ctrlID="ClientExtension"
}