/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenX
 *  $VERSION      1.04 (30-11-2007 19:32)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Nexgen controller extension plugin.
 *
 **************************************************************************************************/
class NexgenX extends NexgenPlugin;

var NexgenXLang lng;                    // Language instance to support localization.

// Extra player attributes.
const PA_Score = "score";               // Score / frag count.
const PA_Deaths = "deaths";             // Number of times the player died.
const PA_StartTime = "starttime";       // Time at which the player joined the game.

// Misc constants.
const CMD_SmartCTFToggleStats = "smartctf stats";



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the plugin. Note that if this function returns false the plugin will
 *                be destroyed and is not to be used anywhere.
 *  $RETURN       True if the initialization succeeded, false if it failed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool initialize() {
	local Actor a;
	
	// Load localization support.
	lng = spawn(class'NexgenXLang');
	
	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a new client has been created. Use this function to setup the new
 *                client with your own extensions (in order to support the plugin).
 *  $PARAM        client  The client that was just created.
 *  $REQUIRE      client != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function clientCreated(NexgenClient client) {
	client.addController(class'NexgenXClient', self);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called whenever a player has joined the game (after its login has been accepted).
 *  $PARAM        client  The player that has joined the game.
 *  $REQUIRE      client != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function playerJoined(NexgenClient client) {
	local NexgenXClient xClient;
	
	// Spawn player overlay for the client.
	if (control.gInf.gameState == control.gInf.GS_Playing && !client.bSpectator) {
		xClient = NexgenXClient(client.getController(class'NexgenXClient'.default.ctrlID));
		xClient.setPlayerOverlay();
	}
	
	// Restore saved player data.
	client.player.playerReplicationInfo.score = client.pDat.getFloat(PA_Score, client.player.playerReplicationInfo.score);
	client.player.playerReplicationInfo.deaths = client.pDat.getFloat(PA_Deaths, client.player.playerReplicationInfo.deaths);
	client.player.playerReplicationInfo.startTime = client.pDat.getInt(PA_StartTime, client.player.playerReplicationInfo.startTime);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called if a player has left the server.
 *  $PARAM        client  The player that has left the game.
 *  $REQUIRE      client != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
function playerLeft(NexgenClient client) {
	local NexgenXClient xClient;
	
	// Get extended client controller.
	xClient = NexgenXClient(client.getController(class'NexgenXClient'.default.ctrlID));
	
	// Remove player overlay.
	xClient.setPlayerOverlay(true);
	
	// Store saved player data.
	client.pDat.set(PA_Score, client.player.playerReplicationInfo.score);
	client.pDat.set(PA_Deaths, client.player.playerReplicationInfo.deaths);
	client.pDat.set(PA_StartTime, client.player.playerReplicationInfo.startTime);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the game has started.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function gameStarted() {
	local NexgenClient client;
	local NexgenXClient xClient;
	
	// Spawn player overlays for client that haven't received one yet.
	for (client = control.clientList; client != none; client = client.nextClient) {
		xClient = NexgenXClient(client.getController(class'NexgenXClient'.default.ctrlID));
		xClient.setPlayerOverlay();
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the game has ended.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function gameEnded() {
	local NexgenClient client;
	local NexgenXClient xClient;
	
	// Destroy player overlays.
	for (client = control.clientList; client != none; client = client.nextClient) {
		xClient = NexgenXClient(client.getController(class'NexgenXClient'.default.ctrlID));
		xClient.setPlayerOverlay(true);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called to check if the specified message should be send to the given receiver.
 *  $PARAM        sender    The actor that has send the message.
 *  $PARAM        receiver  Pawn receiving the message.
 *  $PARAM        msg       The message that is to be send.
 *  $PARAM        bBeep     Whether or not to make a beep sound once received.
 *  $PARAM        type      Type of the message that is to be send.
 *  $RETURN       True if the message should be send, false if it should be suppressed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool mutatorBroadcastMessage(Actor sender, Pawn receiver, out coerce string msg,
                                      optional bool bBeep, out optional name type) {
	local PlayerReplicationInfo senderPRI;
	local bool bIsSpecMessage;

	// Get sender player replication info.
	if (sender != none && sender.isA('Pawn')) {
		senderPRI = Pawn(sender).playerReplicationInfo;
	}
	
	// Check if we're dealing with a spectator chat message.
	bIsSpecMessage = senderPRI != none && sender.isA('Spectator') &&
	                 left(msg, len(senderPRI.playerName) + 1) ~= (senderPRI.playerName $ ":");
	
	// Check for commands.
	if (bIsSpecMessage && sender == receiver) {
		 handleMsgCommand(PlayerPawn(sender), mid(msg, len(senderPRI.playerName) + 1));
	}

	// Don't suppress this message.
	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called to check if the specified team message should be send to the given
 *                receiver.
 *                is called if a message is send to player.
 *  $PARAM        sender    The actor that has send the message.
 *  $PARAM        receiver  Pawn receiving the message.
 *  $PARAM        pri       Player replication info of the sending player.
 *  $PARAM        s         The message that is to be send.
 *  $PARAM        type      Type of the message that is to be send.
 *  $PARAM        bBeep     Whether or not to make a beep sound once received.
 *  $RETURN       True if the message should be send, false if it should be suppressed.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function bool mutatorTeamMessage(Actor sender, Pawn receiver, PlayerReplicationInfo pri,
                                 coerce string s, name type, optional bool bBeep) {
	// Check for commands.
	if (sender != none && sender.isA('PlayerPawn') && sender == receiver &&
	    (type == 'Say' || type == 'TeamSay')) {
		handleMsgCommand(PlayerPawn(sender), s);
	}
	
	// Don't suppress this message.
	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Handles a potential command message.
 *  $PARAM        sender  PlayerPawn that has send the message in question.
 *  $PARAM        msg     Message send by the player, which could be a command.
 *  $REQUIRE      sender != none
 *  $RETURN       True if the specified message is a command, false if not.
 *
 **************************************************************************************************/
function bool handleMsgCommand(PlayerPawn sender, string msg) {
	local string cmd;
	local bool bIsCommand;
	
	cmd = class'NexgenUtil'.static.trim(msg);
	bIsCommand = true;
	switch (cmd) {
		case "!stats": level.game.baseMutator.mutate(CMD_SmartCTFToggleStats, sender); break;

		// Not a command.
		default: bIsCommand = false;
	}
	
	return bIsCommand;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	pluginName="Nexgen extension pack"
	pluginAuthor="Zeropoint"
	pluginVersion="1.02 build 1009"
}