/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenXClient
 *  $VERSION      1.10 (15-3-2008 17:49)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  Extension pack client controller class. This class is the base of the clientside
 *                support for the extension plugin.
 *
 **************************************************************************************************/
class NexgenXClient extends NexgenClientController;

#exec OBJ LOAD FILE=Announcer

var NexgenX xControl;                   // Extension controller
var NexgenXLang lng;                    // Language instance to support localization.
var NexgenXConfig xConf;                // Plugin configuration.

var NexgenXRCPMapSwitch mapSwitchTab;   // Map switch control panel tab.

var NexgenXPlayerOverlay playerOverlay; // Player overlay skin.

var bool bNetWait;                      // Client is waiting for initial replication.

// Map list control.
var bool bSendingMapList;               // Whether the server is currently sending the map list.
var bool bMapListSend;                  // Whether the map list has been sended to the client.
var string firstMap;                    // Filename of the first map on the server.
var string lastMap;                     // Filename of the last map send to the client.
var string maps[1024];                  // Maps available on the server.
var int mapCount;                       // Number of maps available.

// Dynamic control info.
var int lastCountDown;                  // Last known value of gInf.countDown.
var bool bGameStartingAnnounced;        // Has the game starting state been announced yet?

// Config replication control.
var int lastConfigUpdateCount[3];       // Last received config update count per config type.
var int configChecksum[3];				// Checksum to wait for.
var byte bWaitingForNewConfig[3];       // Whether the client is waiting for the configuration.

// Controller settings.
const timerFreq = 5.0;                  // Timer tick frequency.

// General constants.
const maxStringSize = 255;              // Maximum size of strings replicated over the net.
const separator = ",";                  // Character used to seperate elements in a list.
const startCountDownWait = 2;           // Second to wait before starting the count down announcer.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {
	reliable if (role == ROLE_Authority) // Replicate to client...
		// Variables.
		xConf,
		
		// Functions.
		receiveMapListPart, nexgenXConfigChanged;
		
	reliable if (role == ROLE_SimulatedProxy) // Replicate to server...
		// Variables.	
	
		// Functions.
		requestMapList, doMapSwitch, setGeneralSettings, setFullServerRedirSettings;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Prepares the NexgenXClient instance for usage.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function preBeginPlay() {
	
	// Client side actions.
	if (role == ROLE_SimulatedProxy) {
		// Load localization support.
		lng = spawn(class'NexgenXLang');
		
		// Wait for initial replication to complete.
		bNetWait = true;
		
		// Enable timer.
		setTimer(1.0 / timerFreq, true);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Time critical event detection loop.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function tick(float deltaTime) {
	
	// Client side actions.
	if (role == ROLE_SimulatedProxy && client != none && !client.bNetWait) {
		
		// Game starting count down ticked?
		if (client.gInf.gameState == client.gInf.GS_Starting && lastCountDown != client.gInf.countDown) {
			lastCountDown = client.gInf.countDown;
			
			// Play countdown announcer sound?
			if (!bNetWait && xConf.enableStartAnnouncer) {
				if (1 <= lastCountDown && lastCountDown <= 10 &&
				    lastCountDown <= client.sConf.startTime - startCountDownWait) {
					//ChallengeHUD(client.player.myHUD).tellTime(16 - lastCountDown);
					client.player.receiveLocalizedMessage(class'Botpack.TimeMessage', 16 - lastCountDown);
				} else if (!bGameStartingAnnounced) {
					bGameStartingAnnounced = true;
					client.player.clientPlaySound(sound'Announcer.Prepare', , true);
				}
			}
		}
		
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the client controller. This function is automatically called after
 *                the critical variables have been set, such as the client variable.
 *  $PARAM        creator  The Actor that has added the controller to the client.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function initialize(optional Actor creator) {
	xControl = NexgenX(creator);
	lng = xControl.lng;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called by the client when it wants to receive the list of maps that are available
 *                on the server.
 *  $ENSURE       bSendingMapList || bMapListSend
 *
 **************************************************************************************************/
function requestMapList() {
	
	// Check if map list is already being send or has already been send.
	if (bSendingMapList || bMapListSend) {
		// It is and there's no need to send things again.
		return;
	}
	
	// Start sending the map list to the client.
	bSendingMapList = true;
	setTimer(1.0 / timerFreq, true);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Timer tick function. On the server side the timer tick is responsible for sending
 *                the map list to the client if bSendingMapList is set to true.
 *
 **************************************************************************************************/
simulated function timer() {
	local string mapList;
	local string nextMap;
	local bool bMapListComplete;
	local bool bMapListFull;
	
	// Client side actions.
	if (role == ROLE_SimulatedProxy) {
		
		// Wait for initial replication.
		if (client != none && !client.bNetWait && self.bNetWait) {
			if (xConf != none && xConf.updateCount > 0 &&
			    xConf.generalSettingsChecksum == xConf.calcGeneralSettingsChecksum() &&
			    xConf.fullServerRedirectChecksum == xConf.calcFullServerRedirectChecksum()) {
				bNetWait = false;
				clientControllerInitialize();
			}
		}
		
		// Check for config updates.
		if (!bNetWait) {
			checkConfigUpdate();
		}

	// Server side actions.
	} else { 
	
		// Send next part of the map list.
		if (bSendingMapList) {
			
			// Just starting to send the map list?
			if (firstMap == "") {
				// First map hasn't been determined yet, so yes.
				firstMap = getMapName("", "", 0);
				mapList = firstMap;
				lastMap = firstMap;
			}
			
			// Get next map to send.
			nextMap = getMapName("", lastMap, 1);
			
			// Continue adding maps the map list till the string get too large.
			while (!bMapListComplete && !bMapListFull) {
				
				if (nextMap ~= firstMap) {
					// Back at the first map, so we're done.
					bMapListComplete = true;
				} else if (mapList == "")  {
					// Map list is empty, so we can safely add the next map.
					mapList = nextMap;
					lastMap = nextMap;
					nextMap = getMapName("", lastMap, 1);
				} else if (len(mapList) + len(separator) + len(nextMap) > maxStringSize) {
					// Can't add the next map without exceeding the maximum string size.
					bMapListFull = true;
				} else {
					// There's still enough space to add the next map.
					mapList = mapList $ separator $ nextMap;
					lastMap = nextMap;
					nextMap = getMapName("", lastMap, 1);
				}
			}
			
			// Send the map list.
			receiveMapListPart(bMapListComplete, mapList);
			
			// Check if sending has been completed.
			if (bMapListComplete) {
				bSendingMapList = false;
				bMapListSend = true;
				setTimer(0.0, false);
			}
		}
		
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when the client controller has completed its initial replication and is
 *                ready to initialize itself.
 *
 **************************************************************************************************/
simulated function clientControllerInitialize() {
	// Add control panel tabs.
	if (client.hasRight(client.R_MatchAdmin) && xConf.enableMapSwitch) {
		requestMapList();
		mapSwitchTab = NexgenXRCPMapSwitch(client.mainWindow.mainPanel.addPanel(lng.mapSwitchTabTxt, class'NexgenXRCPMapSwitch', , "game"));
	}
	if (client.hasRight(client.R_ServerAdmin)) {
		client.addPluginConfigPanel(class'NexgenXRCPSettings');
		client.addPluginConfigPanel(class'NexgenXRCPFullServerRedir');
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called on the client when a new part of the map list has been received.
 *  $PARAM        bLastPart    Indicates whether this is the last part of the map list.
 *  $PARAM        mapListPart  String containing a part of the map list.
 *
 **************************************************************************************************/
simulated function receiveMapListPart(bool bLastPart, string mapListPart) {
	local string remaining;
	local string mapName;
	
	remaining = mapListPart;
	while (remaining != "" && mapCount < arrayCount(maps)) {
		class'NexgenUtil'.static.split(remaining, mapName, remaining);
		maps[mapCount++] = mapName;
	}
	
	if (bLastPart) {
		mapSwitchTab.notifyMapListAvailable();	
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Causes the server to load the specified map with the given game type and mutators.
 *  $PARAM        mapName   File name of the map that is to be loaded.
 *  $PARAM        gameType  The game type to use when the new map is loaded.
 *  $PARAM        mutators  List of mutators that are to be loaded.
 *
 **************************************************************************************************/
function doMapSwitch(string mapName, byte gameType, string mutators) {
	local string remaining;
	local string indexStr;
	local int index;
	local string mutatorClass;
	local string mutatorClasses;
	local string temp;
	local string gameClass;
	local string bootURL;
	local string mapNameNoExt;
	
	// Preliminary checks.
	if (!client.hasRight(client.R_MatchAdmin)) {
		return;
	}
	
	// Get game type.
	if (0 <= gameType && gameType < arrayCount(client.sConf.gameTypeInfo)) {
		class'NexgenUtil'.static.split(client.sConf.gameTypeInfo[gameType], gameClass, temp);
	} else {
		gameClass = "";
	}
	
	// Get mutator classes.
	remaining = mutators;
	while (remaining != "") {
		class'NexgenUtil'.static.split(remaining, indexStr, remaining);
		index = int(indexStr);
		if (0 <= index && index < arrayCount(client.sConf.mutatorInfo)) {
			class'NexgenUtil'.static.split(client.sConf.mutatorInfo[index], mutatorClass, temp);
			if (mutatorClasses == "") {
				mutatorClasses = mutatorClass;
			} else {
				mutatorClasses = mutatorClasses $ separator $ mutatorClass;
			}
		}
	}
	
	// Assemble URL.
	bootURL = mapName $ "?game=" $ gameClass $ "?mutator=" $ mutatorClasses;
	level.serverTravel(bootURL, false);
	
	// Announce event.
	mapNameNoExt = left(mapName, instr(mapName, "."));
	control.broadcastMsg(lng.adminMapSwitchMsg, client.playerName, mapNameNoExt, , , client.player.playerReplicationInfo);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Adds or removes the player overlay skin for this client.
 *  $PARAM        bRemove  Whether the overlay skin is to be removed.
 *  $ENSURE       bRemove ? new.playerOverlay == none : new.playerOverlay != none
 *
 **************************************************************************************************/	
function setPlayerOverlay(optional bool bRemove) {
	
	// Remove overlay skin?
	if (bRemove && playerOverlay != none) {
		// Yes.
		playerOverlay.destroy();
		playerOverlay = none;
	
	} else if (!bRemove && playerOverlay == none && !client.bSpectator) {
		// No, add overlay skin.
		playerOverlay = spawn(class'NexgenXPlayerOverlay', client.player, ,
		                      client.player.location, client.player.rotation);
		playerOverlay.client = client;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Notifies the client that the server configuration has been updated. The new
 *                settings might not yet have been replicated, so the client has to wait for the
 *                replication to complete. Once the replication is completed an event will be
 *                triggered to signal the GUI and other client controllers that the new settings
 *                are available, see checkConfigUpdate(). Note this function is similar to the
 *                configChanged() function in NexgenClient.
 *  $PARAM        configType   Type of settings that have been changed.
 *  $PARAM        updateCount  Config update number for the new settings.
 *  $PARAM        checksum     Checksum of the new settings.
 *  $REQUIRE      0 <= configType && configType < arrayCount(configChecksum) && updateCount >= 0
 *
 **************************************************************************************************/
simulated function nexgenXConfigChanged(byte configType, int updateCount, int checksum) {
	if (updateCount > lastConfigUpdateCount[configType]) {
		lastConfigUpdateCount[configType] = updateCount;
		configChecksum[configType] = checksum;
		bWaitingForNewConfig[configType] = byte(true);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether some of the configuration was updated and if the new configuration
 *                has been replicated yet to the client. If this is the case an event will be
 *                signalled on the client to notify the system that the new configuration is
 *                available.
 *
 **************************************************************************************************/
simulated function checkConfigUpdate() {
	local int index;
	
	for (index = 0; index < arrayCount(configChecksum); index++) {
		if (bool(bWaitingForNewConfig[index]) &&
		    xConf.updateCount >= lastConfigUpdateCount[index] &&
		    configChecksumMatch(index)) {
		    
			bWaitingForNewConfig[index] = byte(false);
			
			client.notifyEvent(xConf.EVENT_NexgenXConfigChanged, string(index));
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether the configuration checksum stored in the configChecksum array
 *                matches the checksum for the current data.
 *  $PARAM        configType  The configuration type for which the checksum is to be matched.
 *  $RETURN       True when both checksums match, false if not.
 *
 **************************************************************************************************/
simulated function bool configChecksumMatch(byte configType) {
	switch (configType) {
		case xConf.CT_GeneralSettings:      return configChecksum[configType] == xConf.calcGeneralSettingsChecksum();
		case xConf.CT_FullServerRedirect:   return configChecksum[configType] == xConf.calcFullServerRedirectChecksum();
		//case xConf.CT_ServerPackageManager: return configChecksum[configType] == xConf.calcServerPackageManagerChecksum();
		default:                            return false;
	}
	
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Changes the general Nexgen extension plugin settings.
 *  $PARAM        enableOverlaySkin     Whether the player overlay skin effect is enabled.
 *  $PARAM        enableMapSwitch       Whether the map switch tab is enabled.
 *  $PARAM        enableStartAnnouncer  Whether the game starting voice announcer is enabled.
 *
 **************************************************************************************************/
function setGeneralSettings(bool enableOverlaySkin, bool enableMapSwitch, bool enableStartAnnouncer) {
	// Check rights.
	if (!client.hasRight(client.R_ServerAdmin)) {
		return;
	}
	
	// Safe settings.
	xConf.enableOverlaySkin = enableOverlaySkin;
	xConf.enableMapSwitch = enableMapSwitch;
	xConf.enableStartAnnouncer = enableStartAnnouncer;
	xConf.saveConfig();
	
	// Notify system.
	xControl.signalConfigUpdate(xConf.CT_GeneralSettings);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Changes the full server redirect settings.
 *  $PARAM        enableFullServerRedirect  Whether full server redirect is enabled.
 *  $PARAM        fullServerRedirectMsg     Message to display when the server is full.
 *  $PARAM        redirectServerName1       First redirect server name.
 *  $PARAM        redirectServerName2       Second redirect server name.
 *  $PARAM        redirectServerName3       Third redirect server name.
 *  $PARAM        redirectURL1              First redirect server address/URL.
 *  $PARAM        redirectURL2              Second redirect server address/URL.
 *  $PARAM        redirectURL3              Third redirect server address/URL.
 *
 **************************************************************************************************/
function setFullServerRedirSettings(bool enableFullServerRedirect, string fullServerRedirectMsg,
                                    string redirectServerName1, string redirectServerName2,
                                    string redirectServerName3, string redirectURL1,
                                    string redirectURL2, string redirectURL3) {

	// Check rights.
	if (!client.hasRight(client.R_ServerAdmin)) {
		return;
	}
	
	// Safe settings.
	xConf.enableFullServerRedirect = enableFullServerRedirect;
	xConf.fullServerRedirectMsg = left(fullServerRedirectMsg, 250);
	xConf.redirectServerName[0] = left(redirectServerName1, 24);
	xConf.redirectServerName[1] = left(redirectServerName2, 24);
	xConf.redirectServerName[2] = left(redirectServerName3, 24);
	xConf.redirectURL[0] = left(redirectURL1, 64);
	xConf.redirectURL[1] = left(redirectURL2, 64);
	xConf.redirectURL[2] = left(redirectURL3, 64);
	xConf.saveConfig();
	
	// Notify system.
	xControl.signalConfigUpdate(xConf.CT_FullServerRedirect);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	ctrlID="ClientExtension"
}