/***************************************************************************************************
 *
 *  NSC. Nexgen Server Controller by Zeropoint.
 *
 *  $CLASS        NexgenXConfig
 *  $VERSION      1.02 (14-3-2008 0:16)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  NexgenX configuration container/replication class. This class contains the 
 *                settings for the extension plugin.
 *
 **************************************************************************************************/
class NexgenXConfig extends ReplicationInfo;

var NexgenX xControl;                             // NexgenX plugin.

// Special settings.
var config int lastInstalledVersion;              // Last installed version of the plugin.

// General settings.
var int generalSettingsChecksum;                  // Checksum for the general settings variables.
var config bool enableOverlaySkin;                // Enable the player overlay skin effect.
var config bool enableMapSwitch;                  // Whether map switch is available.
var config bool enableStartAnnouncer;             // Play a voice announcer when the game starts.
var config bool enableAntiSpam;                   // Enable message spam detection.

// Full server redirect settings.
var int fullServerRedirectChecksum;               // Check for the full server redirect variables.
var config bool enableFullServerRedirect;         // Whether full server redirect is enabled.
var config string fullServerRedirectMsg;          // Message to display when the server is full.
var config string redirectServerName[3];          // Names of the alternate servers.
var config string redirectURL[3];                 // Redirect URL for the servers.

// Data transfer control.
var int updateCount;                              // Number of times the settings were updated.

// Events.
const EVENT_NexgenXConfigChanged = "nexgenx_config_changed";

// Config types.
const CT_GeneralSettings = 0;                     // General settings config type.
const CT_FullServerRedirect = 1;                  // Full server redirect settings config type.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {

	reliable if (role == ROLE_Authority)
		// General settings.
		generalSettingsChecksum, enableOverlaySkin, enableMapSwitch, enableStartAnnouncer,
		enableAntiSpam,
		
		// Full server redirect settings.
		fullServerRedirectChecksum, enableFullServerRedirect, fullServerRedirectMsg,
		redirectServerName, redirectURL,
		
		// Data transfer control.
		updateCount;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Calculates the checksum for the general settings variables.
 *  $RETURN       The checksum for the general settings variables.
 *
 **************************************************************************************************/
simulated function int calcGeneralSettingsChecksum() {
	local int checksum;
	
	checksum = int(enableOverlaySkin)               |
	           int(enableMapSwitch)            << 1 |
	           int(enableStartAnnouncer)       << 2 |
	           int(enableAntiSpam)             << 3 ;
	
	return checksum;
}




/***************************************************************************************************
 *
 *  $DESCRIPTION  Calculates the checksum for the full server redirect settings variables.
 *  $RETURN       The checksum for the full server redirect settings variables.
 *
 **************************************************************************************************/
simulated function int calcFullServerRedirectChecksum() {
	local int checksum;
	
	checksum = int(enableFullServerRedirect)     +
	           stringHash(fullServerRedirectMsg) +
	           stringHash(redirectServerName[0]) +
	           stringHash(redirectServerName[1]) +
	           stringHash(redirectServerName[2]) +
	           stringHash(redirectURL[0])        +
	           stringHash(redirectURL[1])        +
	           stringHash(redirectURL[2])        ;
	
	return checksum;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Updates the checksum of the general settings variables.
 *  $ENSURE       generalSettingsChecksum == calcGeneralSettingsChecksum()
 *  $RETURN       The new checksum of the general settings variables.
 *
 **************************************************************************************************/
function int updateGeneralSettingsChecksum() {
	generalSettingsChecksum = calcGeneralSettingsChecksum();
	return generalSettingsChecksum;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Updates the checksum of the full server redirect settings variables.
 *  $ENSURE       fullServerRedirectChecksum == calcFullServerRedirectChecksum()
 *  $RETURN       The new checksum of the full server redirect settings variables.
 *
 **************************************************************************************************/
function int updateFullServerRedirectChecksum() {
	fullServerRedirectChecksum = calcFullServerRedirectChecksum();
	return fullServerRedirectChecksum;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Updates the checksums of all replicated variables.
 *  $ENSURE       new.updateCount == old.updateCount + 1 &&
 *                generalSettingsChecksum == calcGeneralSettingsChecksum() &&
 *                fullServerRedirectChecksum == calcFullServerRedirectChecksum() &&
 *                serverPackageManagerChecksum == calcServerPackageManagerChecksum()
 *
 **************************************************************************************************/
function updateAllChecksums() {

	// Update checksums.
	updateGeneralSettingsChecksum();
	updateFullServerRedirectChecksum();
	
	// Set new update count.
	updateCount++;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Gives the integer hash value of a string.
 *  $PARAM        str  The string that is to be hashed.
 *  $RETURN       The hash value of the given string.
 *
 **************************************************************************************************/
static function int stringHash(coerce string str) {
	local int hash;
	
	if (str != "") {
		hash = 29789 * asc(left(str, 1))              +
		         953 * asc(mid(str, len(str) / 2, 1)) +
		          31 * asc(right(str, 1))             +
		       len(str);
	}
	
	return hash;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Loads the plugins configuration.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function preBeginPlay() {
	
	xControl = NexgenX(owner);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Automatically installs the extension plugin.
 *  $ENSURE       lastInstalledVersion >= xControl.versionNum
 *
 **************************************************************************************************/
function install() {
	
	if (lastInstalledVersion < 105) installVersion105();
	if (lastInstalledVersion < 106) installVersion106();
	//if (lastInstalledVersion < 107) installVersion107();
	//if (lastInstalledVersion < 108) installVersion108();
	//if (lastInstalledVersion < 109) installVersion109();
	// etc.
	
	if (lastInstalledVersion < xControl.versionNum) {
		lastInstalledVersion = xControl.versionNum;
	}
	saveConfig();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Automatically installs version 105 of the Nexgen extension plugin.
 *
 **************************************************************************************************/
function installVersion105() {
	enableOverlaySkin = true;
	enableMapSwitch = true;
	enableStartAnnouncer = true;
	enableFullServerRedirect = false;
	fullServerRedirectMsg = xControl.lng.defaultFullServerRedirMsg;
	redirectServerName[0] = "Server 1";
	redirectURL[0] = "unreal://127.0.0.1:7777/";
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Automatically installs version 106 of the Nexgen extension plugin.
 *
 **************************************************************************************************/
function installVersion106() {
	enableAntiSpam = true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Validates the configuration. Problems with the Nexgen extension plugin
 *                configuration will be automatically repaired.
 *  $RETURN       True if the configuration was valid, false if there were one or more configuration
 *                corruptions.
 *  $ENSURE       imply(old.validate(), new.validate())
 *
 **************************************************************************************************/
function bool validate() {
	local bool bInvalid;
	
	bInvalid = bInvalid || fixStrLen(fullServerRedirectMsg, 250);
	bInvalid = bInvalid || fixStrLen(redirectServerName[0], 24);
	bInvalid = bInvalid || fixStrLen(redirectServerName[1], 24);
	bInvalid = bInvalid || fixStrLen(redirectServerName[2], 24);
	bInvalid = bInvalid || fixStrLen(redirectURL[0], 64);
	bInvalid = bInvalid || fixStrLen(redirectURL[1], 64);
	bInvalid = bInvalid || fixStrLen(redirectURL[2], 64);
	
	return !bInvalid;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the configuration.
 *
 **************************************************************************************************/
function initialize() {
	
	// Last thing to do is to make sure all the checksums are up to date.
	updateAllChecksums();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Fixes the length of a string. This function makes sure the length of a given
 *                string doesn't exceed the specified maximum length.
 *  $PARAM        str  The string of which the length has to be checked.
 *  $PARAM        maxLen  The maximum length of the string.
 *  $REQUIRE      maxLen >= 0
 *  $RETURN       True if the length of the string was changed, false otherwise.
 *  $ENSURE       len(new.str) <= maxLen
 *
 **************************************************************************************************/
function bool fixStrLen(out string str, int maxLen) {
	if (len(str) > maxLen) {
		str = left(str, maxLen);
		return true;
	} else {
		return false;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Fixes the value of a given integer variable. Calling this function will ensure
 *                that the value of the variable will be in the specified domain.
 *  $PARAM        intVar      The integer variable whose value is to be checked.
 *  $PARAM        lowerBound  Lower bound on the range of the variable.
 *  $PARAM        upperBound  Upperbound bound on the range of the variable.
 *  $RETURN       True if value of the integer variable was changed, false otherwise.
 *  $ENSURE       lowerBound <= intVar && intVar <= upperBound
 *
 **************************************************************************************************/
function bool fixIntRange(out int intVar, int lowerBound, int upperBound) {
	if (intVar < lowerBound) {
		intVar = lowerBound;
		return true;
	} else if (intVar > upperBound) {
		intVar = upperBound;
		return true;
	} else {
		return false;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Fixes the value of a given byte variable. Calling this function will ensure
 *                that the value of the variable will be in the specified domain.
 *  $PARAM        byteVar     The byte variable whose value is to be checked.
 *  $PARAM        lowerBound  Lower bound on the range of the variable.
 *  $PARAM        upperBound  Upperbound bound on the range of the variable.
 *  $RETURN       True if value of the byte variable was changed, false otherwise.
 *  $ENSURE       lowerBound <= byteVar && byteVar <= upperBound
 *
 **************************************************************************************************/
function bool fixByteRange(out byte byteVar, byte lowerBound, byte upperBound) {
	if (byteVar < lowerBound) {
		byteVar = lowerBound;
		return true;
	} else if (byteVar > upperBound) {
		byteVar = upperBound;
		return true;
	} else {
		return false;
	}
}
