/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLConfig
 *  $VERSION  1.02
 *  $AUTHOR   Daan 'Defrost' Scheerens  original version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION  SLV Configuration container class. Also provides code to replicate some of the
 *                settings to the clients.
 *  $SINCE        SLV203
 *
 **************************************************************************************************/
class SLConfig extends ReplicationInfo config(SLV204);

enum EWarStatX { WH_Auto, WH_Armed, WH_Disarmed };
enum ESyncModeX { SN_NoSync, SN_ClientSync, SN_ServerSync };
enum EFireModeX { FM_Single, FM_Burst, FM_Auto };
enum ECarryMode { CM_Off, CM_Normal, CM_Compatability };

// Server settings.
var config int    maxRockets;           // Maximum amount of SLV rockets that can be carried.
var config float  teamDamage;           // Multiplier for damage caused by a teammate to SLV rockets.
var config int    initialFuelCores;     // Amount of fuel cores initially given to a player.
var config int    fuelCoresPerLauncher; // Fuel cores given to a player for each launcher picked up.
var config int    maxFuelCores;         // Max number of fuel cores to carry.
var config float  maxFuel;              // Default amount of fuel for rockets and cores.
var config bool   replacePacks;         // Replace health packs with fuel pods.
var config bool   replaceVials;         // Replace health vials with fuel pods.
var config bool   randomSLVPlacement;   // Spawn random SLV launchers over time.
var config bool   noYJ;                 // Remove the Yellow Jacket from the game.
var config bool   giveMaxAmmo;          // The SLV and YJ weapons get their maximum of ammo.
var config int    maxSidearmClips;      // Max number of YJ clips to carry.
var config int    roundsPerClip;        // Number of bullets per clip.
var config EFireModeX defSidearmMode;   // Default fire mode of the yellow jacket.
var config float  sidearmDamage;        // Hit damage caused by Yellow Jacket fire.
var config float  shockwaveSize;        // Size of the shockwave created by an exploding SLV rocket.
var config EWarStatX  warheadStatus;    // Initial warhead status of SLV rockets.
var config ESyncModeX syncMode;         // Rocket synchronisation method to use.
var config bool   noArmedSwitch;        // Players can't switch to armed warhead state.
var config bool   canOverrideWarStat;   // Players can't change the warhead state.
var config bool   useKillMsgFix;        // Use the kill message fix.
var config bool   killMsgFixSLVOnly;    // Limit the kill message fix to SLV rockets only.
var config bool   enableBotSupport;     // Enable bot support for SLV rockets.
var config float  minSpeed;             // Rocket velocity at minimum throttle.
var config float  fullSpeed;            // Rocket velocity at maximum throttle.
var config float  maxSpeed;             // Rocket velocity when the afterburner is enabled.
var config float  rocketArmor;          // Amount of damage the rocket hull can withstand.
var config bool   enableBoosters;       // Enable the use of boosters for SLV rockets.
var config float  boosterFuel;          // Amount of fuel for the boosters.
var config float  boosterSpeed;         // Extra speed gained by using the boosters.
var config bool   coloredDetonations;   // Use team colored detonations.
var config bool   noPilotTeamDamage;    // Do unpiloted slv rockets take team damage?
var config string excludedActors[16];   // Actors excluded from being replaced by StrangeArena.
var config string customSLVSkins[8];    // Customized rocket skins.
var config ECarryMode pilotCarryMode;   // Is the pilot of a SLV rocket allowed to carry the flag?

// Client settings.
var config bool  showKeys;              // Display an overview of the SLV controls.
var config bool  showRearView;          // Show the rear view mirror when flying on a rocket.
var config bool  disableContrails;      // Don't show the rocket contrails.
var config bool  contrailRotate;        // Contrail sections slowly rotate.
var config bool  contrailViewAlign;     // Contrail sections automatically align to the players view.
var config float contrailLifeTime;      // Life time of the contrail sections.
var config bool  disableLaunchScream;   // Disable the scream sound played when launching a rocket.
var config bool  showDamageSparks;      // Spawn spark particles when a SLV rocket takes damage.

// Replication checking.
var string replicationCheck;            // Used to detect when the initial replication is complete.
const replicationSignal = "NetReady";   // Value that signals completion of the initial replication.
const replicationCheckFreq = 20.0;      // Frequency of the replication checks.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {
	reliable if (role == ROLE_Authority && bNetInitial)
		maxFuelCores, maxFuel, roundsPerClip, shockwaveSize, warheadStatus, syncMode, minSpeed,
		fullSpeed, maxSpeed, rocketArmor, enableBoosters, boosterFuel, boosterSpeed, defSidearmMode,
		customSLVSkins, replicationCheck;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes this object. Applies the settings for client or server.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function preBeginPlay() {
	if (role == ROLE_Authority) { 
		applyServerSettings(); 
		applyReplicatedServerSettings();
		replicationCheck = replicationSignal;
	} 
	
	if (level.netMode != NM_DedicatedServer) { 
		applyClientSettings(); 
	}
	
	if (level.netMode == NM_Client) { 
		setTimer(1.0 / replicationCheckFreq, true);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Timer tick. During each tick the client checks if the replicated server variables
 *                have been received. If this is the case the settings are applied and the timer is
 *                stopped.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function timer() {
	// Check if initial replication info has been received.
	if (replicationCheck == replicationSignal) {
		setTimer(0.0, false);
		applyReplicatedServerSettings();
		//destroy(); // No longer need this object.
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Applies the server settings.
 *
 **************************************************************************************************/
function applyServerSettings() {
	class'StrangeAmmo'.default.configMaxAmmo = maxRockets;
	class'SLSidearmAmmo'.default.maxClips = maxSidearmClips;
	class'SLSidearm'.default.hitDam = sidearmDamage;
	class'StrangeShell'.default.teamDamage = teamDamage;
	class'StrangeShell'.default.bDisableArmedWH = noArmedSwitch;
	class'StrangeShell'.default.bCanOverride = canOverrideWarStat;
	class'SLKillMsgFix'.default.bEnableSLVCheck = killMsgFixSLVOnly;
	class'SLSkyNet'.default.bBotSupport = enableBotSupport;
	class'StrangeShell'.default.bTeamDetClass = coloredDetonations;
	class'StrangeShell'.default.bNoPilotTeamDam = noPilotTeamDamage;
	class'StrangeShell'.default.bPilotCantCarryFlag = pilotCarryMode != CM_Off;
	class'SLKongLauncher'.default.numFuelCoresOnPickup = fuelCoresPerLauncher;
	class'SLFuelcore'.default.configMaxAmmo = maxFuelCores;
	class'SLFuelcore'.default.maxAmmo = maxFuelCores;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Applies the server settings which the client also has to know.
 *  $REQUIRE      replicationCheck == replicationSignal
 *
 **************************************************************************************************/
simulated function applyReplicatedServerSettings() {
	default.maxFuelCores = maxFuelCores;
	
	default.defSidearmMode = defSidearmMode;
	
	default.roundsPerClip = roundsPerClip;
	
	default.shockwaveSize = shockwaveSize;
	
	default.syncMode = syncMode;
	default.maxFuel = maxFuel;
	default.warheadStatus = warheadStatus;
	// $ADD(SLV203)  Ability to change the cruising speeds of SLV rockets.
	default.minSpeed = minSpeed;
	default.fullSpeed = fullSpeed;
	if (enableBoosters) {
		default.maxSpeed = maxSpeed + boosterSpeed;
	} else {
		default.maxSpeed = maxSpeed;
	}
	// $ADD(SLV203)  Option to equip rockets with an additional booster.
	default.enableBoosters = enableBoosters;
	default.boosterFuel = boosterFuel;
	default.boosterSpeed = boosterSpeed;
	// $ADD(SLV203)  Configurable rocket armor.
	default.rocketArmor = rocketArmor;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Applies the client settings.
 *
 **************************************************************************************************/
simulated function applyClientSettings() {

	/*	
	default.contrailLifeTime = contrailLifeTime;
	default.contrailViewAlign = contrailViewAlign;
	default.contrailRotate = contrailRotate;
	
	default.showKeys = showKeys;
	default.showRearView = showRearView;

	default.disableLaunchScream = disableLaunchScream;
	default.showDamageSparks = showDamageSparks;
	default.disableContrails = disableContrails;
	*/
}

// $END_CLASSIFIED

defaultproperties {
	netPriority=8.0
	remoteRole=ROLE_SimulatedProxy
	maxRockets=6
	teamDamage=0.0
	initialFuelCores=0
	fuelCoresPerLauncher=0
	maxFuelCores=5
	maxFuel=300.0
	replacePacks=false
	replaceVials=false
	randomSLVPlacement=false
	noYJ=false
	giveMaxAmmo=false
	maxSidearmClips=10
	roundsPerClip=60
	sidearmDamage=7
	defSidearmMode=FM_Single
	shockwaveSize=10.0
	warheadStatus=WH_Auto
	syncMode=SN_ServerSync
	noArmedSwitch=false
	showKeys=true
	showRearView=false
	disableContrails=false
	contrailRotate=false
	contrailViewAlign=false
	contrailLifeTime=20.0
	disableLaunchScream=false
	canOverrideWarStat=true
	useKillMsgFix=true
	killMsgFixSLVOnly=true
	enableBotSupport=false
	minSpeed=500.0
	fullSpeed=900.0
	maxSpeed=1350.0
	rocketArmor=100.0
	enableBoosters=false
	boosterFuel=40.0
	boosterSpeed=400.0
	coloredDetonations=false
	showDamageSparks=false
	noPilotTeamDamage=false
	pilotCarryMode=CM_Off
}