/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    StrangeWave
 *  $VERSION  1.16
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *    2006.06.04  Def    -  Added extra failsafe code to prevent Accessed None warnings in
 *                          Engine.GameInfo.Killed() for cases in which the instigator or it's
 *                          weapon is none. All changes have been made in the vaporize() function.
 *
 **************************************************************************************************/
 
class StrangeWave extends ShockWave;

var() Sound detonateSound;
var bool bSpawnedEffects;
var() float meshr;
var() localized string deathStr;
var() localized string deathSelfStr;
var() name dtype;
var SLThermoWave slave;
var bool bSlaveDown;

var Weapon savedWeapon;

simulated function postBeginPlay() {
	local Actor a;
	local vector hl;
	local vector hn;
	
	if (level.netMode != NM_DedicatedServer) {
		slave = spawn(class'SLThermoWave');
		slave.setRotation(rot(0, 0, -32768));
		slave.setLocation(location + vect(0.0, 0.0, 1.0) * 20);
		slave.multiSkins[0] = multiSkins[0];
		bSlaveDown = true;
	}
	
	loopAnim('rotate', 0.7);
	
	super.postBeginPlay();
}

simulated function setRadius() {
	shockSize = cos(lifeSpan) * class'SLConfig'.default.shockwaveSize;
}

simulated function tick(float delta) {
	if (Level.NetMode != NM_DedicatedServer) {
		if (class'SLConfig'.default.shockwaveSize != 0 && !bSpawnedEffects) {
			bSpawnedEffects = True;
			playSound(detonateSound, SLOT_None, 9.0);
			playSound(detonateSound, SLOT_Misc, 9.0);
		}
		
		setRadius();
		
		scaleGlow = lifeSpan;
		ambientGlow = default.ambientGlow * (lifespan / default.lifespan);
		drawScale = shockSize;
		
		if (slave != none) {
			slave.scaleGlow = scaleGlow;
			slave.ambientGlow = slave.default.ambientGlow * (lifespan / default.lifespan);
			slave.drawScale = drawScale;
			if (!bSlaveDown) {
				slave.drawScale *= 0.75;
			}
		}
	}
}

simulated function timer() {
	local int i;
	local vector dir;
	local StrangeExpl exp;
	local SLDamagePuff puff;

	if (level.netMode == NM_DedicatedServer) {
		setRadius();
	}
	
	vaporize();
}

simulated function vaporize() {
	local Actor victim;
	local float damageScale;
	local float dist;
	local float moScale;
	local vector dir;
	local SLUtil U;
	
	local Pawn source;
	local bool bWeaponRestored; 
	local SLKongLauncher kl;
	
	// Save the weapon of the instigator so that we can restore the weapon before calling takeDamage
	// for the victims. Calling takeDamage will call Engine.GameInfo.Killed and cause a null pointer
	// violation when it tries to access the instigators weapon.
	if (savedWeapon == none && instigator != none) {
		savedWeapon = instigator.weapon;
	}
	
	// If the damn weapon is still none, then just use the first SLKongLauncer we see.
	if (savedWeapon == none) {
		foreach allActors(class'SLKongLauncher', kl) {
			savedWeapon = kl;
			break;
		}
	}
	
	foreach visibleCollidingActors(class'Actor', victim, shockSize * meshr, location) {
		if (victim.role == ROLE_Authority) {
			
			dir = victim.location - location;
			dist = fmax(1.0, vsize(dir));
			dir = dir / dist + vect(0.0, 0.0, 0.3);
			
			if ((dist > oldShockDistance) || (dir dot victim.velocity <= 0)) {
				moScale = fmax(0.0, 1100.0 - 1.1 * dist);
				
				if (role == ROLE_Authority && victim.bIsPawn) {
					Pawn(victim).addVelocity(dir * (moScale + 20));
					if (u == none) {
						u = spawn(class'SLUtil', self);
					}
					
					if (victim == instigator) {
						u.setDamageString(deathSelfStr, instigator, victim);
					} else {
						u.setDamageString(deathStr, instigator, victim);
					}
				} else {
					victim.velocity = victim.velocity + dir * (moScale + 20);
				}
				
				// Set the source pawn of the blast. Actually this should always be the instigator,
				// however that guy might be none. In that case we should use the victim itself as
				// the source of the blast, so at least there won't be Accessed None warnings.
				if (instigator != none) {
					source = instigator;
				} else if (victim.isA('Pawn')) {
					source = Pawn(victim);
				} else {
					source = none;
				}
				
				// If the source pawn has no weapon, temporary give him or her one so we won't get
				// warnings when the SpecialDamage string is parsed.
				if (source != none && source.weapon == none) {
					source.weapon = savedWeapon;
					bWeaponRestored = true;
				}
				
				victim.takeDamage(moscale, source, victim.location - 0.5 * (victim.collisionHeight + victim.collisionRadius) * dir, (1000 * dir), dtype);
				
				// Unset temp weapon.
				if (bWeaponRestored) {
					source.weapon = none;
					bWeaponRestored = false;
				}
			}
		}
	}
	
	oldShockDistance = shockSize * meshr;
}

simulated function spawnEffects() {
	// Ignore.
}

simulated function destroyed() {
	local vector loc;

	if (level.netMode != NM_DedicatedServer) {
		if (!level.bDropDetail) {
			spawn(class'SLBlackCloud');
		}
	}
	
	super.destroyed();
}

// $END_CLASSIFIED

defaultproperties {
    detonateSound=Sound'SLV2Sounds.Rocket.detonate'
    meshr=50.00
    deathStr="%o went out in %ks big nuclear fireball."
    deathSelfStr="%k blew %kopself to high hell."
    dtype=SpecialDamage
    LifeSpan=1.00
    Mesh=SkeletalMesh'SLV2Models.thermo'
    AmbientGlow=180
    SoundRadius=255
}
