/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLKillMsgFix
 *  $VERSION  1.04
 *  $AUTHOR   Daan 'Defrost' Scheerens  original version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *    A little mutator that bring the kill messages back to SLV. Those aren't displayed since
 *    SpecialDamage is used as damageType when a player gets killed.
 *  $CHANGES
 *    2006.11.15  Def    -  Added a check to determine whether the scoreKill is called due to a SLV
 *                          killing a player, to prevent double multiLevel increase issues. This
 *                          introduced the following variables: bEnableSLVCheck, registeredKiller
 *                          and registrationTime. Also two new functions have been added:
 *                          isSLVInstigatedKill() and registerSLVKill().
 *
 *    2007.04.20  Def    -  Added a check to ensure there is only one instance of this class active.
 *
 **************************************************************************************************/
class SLKillMsgFix extends Mutator;

var bool  bEnableSLVCheck;       // Check for SLV instigated scoreKill calls?
                                 // Disabling this may cause other weapons to increase the
                                 // multiLevel of a player two times for the same kill.
var Pawn  registeredKiller;      // Player which caused a SLV instigated kill.
var float registrationTime;      // Time at which this event has occurred, used to detect
                                 // outdated registrations.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when an instance of this class is spawned and is to be initialized. Checks
 *                if there is already an existing instance of this class. If so this instance will
 *                be destroyed to ensure only one SLKillMsgFix mutator is active during the game.
 *  $OVERRIDE
 *
 **************************************************************************************************/
function preBeginPlay() {
	local SLKillMsgFix m;
	
	foreach allActors(class'SLKillMsgFix', m) {
		if (m != self) {
			break;
		}
	}
	
	if (m != none) {
		warn("Multiple instances of SLKillMsgFix detected, destroying" @ self @ "...");
		destroy();
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when a player was killed and may manually broadcast kill messages. If
 *                bEnableSLVCheck is enabled this fix first checks if the kill was caused by a
 *                StrangeShell rocket.
 *  $PARAM        killer  the pawn which is responsible for this kill
 *  $PARAM        other   victim of the kill
 *  $OVERRIDE
 *
 **************************************************************************************************/
function scoreKill(Pawn killer, Pawn other) {
	
	// Check whether to use the kill message fix.
	if (killer != none && other != none && killer != other && (!bEnableSLVCheck || isSLVInstigatedKill(killer))) {

		// Display "you were killed by ..." message.
		other.receiveLocalizedMessage(class'VictimMessage', 0, killer.playerReplicationInfo);
		
		// Display "you killed ..." message.
		killer.receiveLocalizedMessage(class'KillerMessagePlus', , killer.playerReplicationInfo, other.playerReplicationInfo);
		
		if (killer.isA('TournamentPlayer')) {
			// Check for double, multi, ultra and monsterrrrrrrr kills.
			if (killer.level.timeSeconds - TournamentPlayer(killer).lastKillTime < 3) {
				TournamentPlayer(killer).multiLevel++;
				killer.receiveLocalizedMessage(class'MultiKillMessage', TournamentPlayer(killer).multiLevel);
			} else {
				TournamentPlayer(killer).multiLevel = 0;
			}
			
			// Update last kill time.
			TournamentPlayer(killer).lastKillTime = killer.level.timeSeconds;
		}
	}
	   
    // Allow next mutator to do its job.
    if (nextMutator != none) {
        nextMutator.scoreKill(killer, other);
    }
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether the kill just made was caused by a StrangeShell rocket. The kill
 *                just made refers to a kill announced to the scoreKill function. Therefore this
 *                function only returns a meaningful answer when its called from that function.
 *  $PARAM        killer the pawn which is responsible for this kill
 *  $REQUIRE      killer != none
 *  $RETURN       true when the kill was caused by a SLV, false otherwise
 *
 **************************************************************************************************/
function bool isSLVInstigatedKill(Pawn killer) {
	return killer == registeredKiller && killer.level.timeSeconds == registrationTime;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Registers a potential SLV caused kill instigated by the specified pawn.
 *  $PARAM        killer the pawn which is responsible for this kill
 *  $REQUIRE      killer != none
 *  $ENSURE       isSLVInstigatedKill(killer)
 *
 **************************************************************************************************/
function registerSLVKill(Pawn killer) {
	registeredKiller = killer;
	registrationTime = killer.level.timeSeconds;
}

// $END_CLASSIFIED

defaultproperties {
	bEnableSLVCheck=true
}