/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLWeapon
 *  $VERSION  1.27
 *  $AUTHOR   Al 'yrns' McElrath        original version
 *            Daan 'Defrost' Scheerens  maintenance
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION
 *  $CHANGES
 *
 *    2006.11.23  Def    -  Added support for UTPureRC7G. The postRender function isn't called when
 *                          UTPure is enabled. The workaround is simply calling it from the
 *                          renderOverlays() function.
 *
 *    2006.11.24  Def    -  Fixed weapon animations not working correctly in netgames. This was
 *                          caused by the fact setHand wasn't declared simulated (a result from
 *                          the extraction of the SLV201 code).
 *
 *    2006.12.15  Def    -  Fixed crosshairs not displayed if the weapon was hidden, caused by an
 *                          invalid condition in renderOverlays.
 *
 *    2007.04.08  Def    -  Removed debug logging code.
 *
 **************************************************************************************************/
class SLWeapon extends TournamentWeapon	abstract;

var bool bFirearm;
var() int hitdam;
var SLClip clip;
var() bool bFakeReload;
var() Class<SLEjectedBrass> brassClass;
var() Class<SLWallhit> wallhitClass;
var bool bSetProjOwner;
var Projectile lastProj;
var Vector spawnProjOffset;

enum EFireMode {
	FM_Single,
	FM_Burst,
	FM_Auto
};

var EFireMode fireMode;
var() bool bAltSwitchMode;
var() int rpm;
var() float burstWait;
var() int burstRounds;
var() int burstCount;
var bool bClear;
var bool bInitFA;

struct RecoilInfo {
	var float base;
	var float limit;
	var float kick;
	var float returnRate;
	var float smoothRate;
	var float pvoScale;
	var float accScale;
};

var() RecoilInfo recoilData;
var Vector recoilv;
var Vector smoothrv;
var bool bAlt;
var() bool bAkimbo;
var SLWeapon slave;
var SLWeapon master;
var bool bSlave;
var bool bSetUp;
var bool bBringingUp;
var() localized string slaveName;
var() byte akimboSwitchPriority;
var() string leftMesh;
var() float slaveWait;
var() float slaveAltWait;
var() float slaveAcc;

struct ASS {
	var() name aname;
	var() float arate;
	var() float tween;
	var() Sound Sound;
	var() float Volume;
	var() ESoundSlot Slot;
	var() bool bSkipOwner;
};

var() ASS assFire;
var() ASS assAltFire;
var() ASS assSelect;
var() ASS assDown;
var() ASS assReload;
var() ASS assIdle;
var() ASS assEmpty;
var() ASS assMode;
var() float idleLapse;
var() float idleRandom;
var() Texture xhair;
var() Texture xhair_xl;
var() bool bNoCenter;
var() Texture MFVariations[6];
var() float paSlide;
var() float paFlashY;
var() float paFlashO;
var() name mimicPriorityOf;
var float retbar;
var string retmsg;
var string retmsg2;
var string retmsg3;
var Font retFont;
var SLStylus yy;
var SLUtil u;

replication {
	reliable if (role == ROLE_Authority && bNetOwner && bAkimbo)
		slave, master, bSlave, bBringingUp;

	reliable if (role == ROLE_Authority && bNetOwner)
		clip;

	reliable if (role == ROLE_Authority)
		remoteReload, updateMaxAmmo, fireMode;
}

simulated function postBeginPlay() {
	local class<Weapon> c;
	
	u = spawn(class'SLUtil', self);
	
	if (mimicPriorityOf != '') {
		c = class<Weapon>(dynamicLoadObject("Botpack." $ string(mimicPriorityOf), class'Class'));
		if (c != none) {
			autoSwitchPriority = c.default.autoSwitchPriority;
			if (bAkimbo && akimboSwitchPriority == -1) {
				akimboSwitchPriority = autoSwitchPriority;
			}
		}
	}
	
	u.loadLeftMeshes();
		
	if (role < ROLE_Authority) {
		return;
	}

	super.postBeginPlay();
}

simulated function SLAmmo ammo() {
	if (ammoType == none) {
		giveAmmo(Pawn(owner));
	}
	return SLAmmo(ammoType);
}

function bool useAmmo(int num){
	local SLAmmo a;

	a = ammo();
	if (a.bClipped) {
		return clip != none && clip.useRound();
	} else {
		return a.useAmmo(num);
	}
}

function giveAmmo(Pawn p) {
	local int max;
	local bool bUpdate;

	if (ammoName == none) {
		return;
	}

	max = class<SLAmmo>(ammoName).default.configMaxAmmo;
	bUpdate = ammoType == none && max > 0;
	ammoType = Ammo(p.findInventoryType(ammoName));
	
	if (ammoType == none) {
		ammoType = spawn(ammoName);
		p.addInventory(ammoType);
		ammoType.becomeItem();
		ammoType.gotoState('Idle2');
		ammo().weaponSpawned();
	}
	
	if (ammo().bClipped) {
		clip = ammo().defaultClip();
		pickupAmmoCount--;
	}
	
	ammoType.addAmmo(pickupAmmoCount);
	
	if (bUpdate) {
		updateMaxAmmo(max);
	}
}

function maxOut() {
	local int max;

	max = class<SLAmmo>(ammoName).default.configMaxAmmo;
	if (max > 0) {
		pickupAmmoCount = max;
	} else {
		pickupAmmoCount = class<SLAmmo>(ammoName).default.maxAmmo;
	}
}

simulated function updateMaxAmmo(int max) {
	if (ammoType != none) {
		ammoType.maxAmmo = max;
	}
	ammoName.default.maxAmmo = max;
}

simulated function int getAmmo() {
	return ammo().getAmmo();
}

simulated function int getMaxAmmo() {
	return ammo().getMaxAmmo();
}

simulated function bool ammoLeft() {
	return getAmmo() > 0;
}

simulated function bool needReload() {
	return ammo().bClipped && clip.empty() && ammoLeft();
}

event float botDesireability(Pawn bot) {
	local SLWeapon has;
	local float desire;

	has = SLWeapon(bot.findInventoryType(class));
	desire = maxDesireability + bot.adjustDesireFor(self);
	
	if (has != none) {
		if ((!bHeldItem || bTossedOut) && bWeaponStay) {
			return 0.0;
		}
		
		if (has.slave != none) {
			if ((respawnTime < 10) && (bHidden || (has.ammoType == none) || (has.getAmmo() < has.getMaxAmmo()))) {
				return 0.0;
			}

			if (has.ammoType == none) {
				return 0.25 * desire;
			}

			if (has.getAmmo() > 0) {
				return fmax(0.25 * desire, has.ammoType.maxDesireability * fmin(1.0, 0.15 * has.getMaxAmmo() / has.getAmmo())); 
			}
		}
	}
	
	if (bot.weapon == none || bot.weapon.AIRating <= 0.4) {
		return 2.0 * desire;
	}
	
	return desire;
}

simulated function xPostRender(Canvas c) {
	local ChallengeHUD hud;

	if (owner != none) {
		hud = ChallengeHUD(PlayerPawn(owner).myHUD);
		if (HUD != none) {
			c.bNoSmooth = false;
			
			if (!bSlave) {
				drawXhair(hud, c);
			}

			setRetMsgs();
			drawRetMsgs(c, 15 * hud.crosshairColor);
			
			// The xPostRender() function of the slave is already called. This happens because
			// renderOverlays also calls itself on the slave weapon.
			/*
			if (slave != none) {
				slave.xPostRender(c);
			}
			*/
		}
	}
}

simulated function drawRetMsgs(Canvas c, Color color) {
	local int x, y, dist;
	local float w, h, xl, yl;

	if (retmsg == " ") {
		return;
	}
	
	if (yy == none) {
		yy = spawn(class'SLStylus', self);
	}
	
	yy.setCanvas(c);

	dist = 48;
	if (bSlave) {
		dist = 56;
	}

	if (retFont == none) {
		retFont = class'SLFontlib'.static.getFont(FS_Slick);
	}
	
	if (retFont == none) {
		return;
	}
	
	c.font = retFont;
	c.style = ERenderStyle.STY_Translucent;
	c.drawColor = color;
	
	c.textSize(retmsg, xl, yl);
	w = xl;
	h = yl;
	c.textSize(retmsg2, xl, yl);
	w = fmax(w, xl);
	w = fmax(w, 50.0);

	if (hand() < 0) {
		x = c.clipx / 2.0 + dist;
	} else {
		x = c.clipx / 2.0 - (dist + w);
	}
	y = c.clipy / 2.0 - 20;

	if (retbar >= 0.0) {
		yy.drawFrame(x, y, w + 2, yl + 2, 1, false);
		c.setPos(x + 1, y + 1);
		c.drawColor = color * 0.8;
		yy.pix(w * retbar, yl);
	}
	
	c.drawColor = color;

	c.setPos(x + 2, y + 2);
	c.drawText(retmsg);
	c.setPos(x + 2, y + yl + 3);
	c.drawText(retmsg2);

	if (retmsg3 != " ") {
		c.textSize(retmsg3, xl, yl);
		c.setPos(x + w - xl + 1, y + 2);
		c.drawText(retmsg3);
	}
}

simulated function setRetMsgs() {
	local SLAmmo A;

	retmsg = " ";
	retmsg2 = " ";
	retmsg3 = " ";
	a = ammo();
	if (a == none || a.bClipped && clip == none) {
		return;
	}
	
	if (a.bClipped) {
		retbar = float(clip.getRounds()) / clip.getMaxRounds();
		retmsg = "rnds " $ string(clip.getRounds());

		if (!bSlave) {
			retmsg2 = "clips " $ a.getClips();
		}
	
	} else {
		retbar = float(getAmmo()) / getMaxAmmo();
		retmsg = "rnds " $ getAmmo();
	}
	
	if (bFirearm && !bSlave) {
		switch (fireMode) {
		case FM_Single:
			retmsg3 = "semi";
			break;

		case FM_Burst:
			retmsg3 = "burst";
			break;

		case FM_Auto:
			retmsg3 = "auto";
			break;
		}
	}

	if (isInState('reloading')) {
		retmsg = "reloading";
		retmsg3 = " ";
	}
}

simulated function drawXhair(ChallengeHUD hud, Canvas c) {
	local float scale;
	local float w;
	local float pickd;

	if (xhair == none) {
		return;
	}

	if (c.clipX < 512) {
		scale = 0.5;
	} else {
		scale = fmax(1.0, int(0.1 + c.clipx / 640.0));
	}

	pickd = level.timeSeconds - HUD.pickupTime;
	if (pickd < 0.4) {
		if (pickd < 0.2) {
			scale *= 1 + 5 * pickd;
		} else {
			scale *= 3 - 5 * pickd;
		}
	}
	
	scale += fmin(1.0, vsize(recoilv) / 1000.0);

	w = 64.0 * scale;

	if (PlayerPawn(owner).handedness == -1) {
		c.setPos(0.503 * (c.clipx - w), 0.504 * (c.clipy - w));
	} else if (PlayerPawn(owner).handedness == 1) {
		c.setPos(0.497 * (c.clipx - w), 0.496 * (c.clipy - w));
	} else {
		c.setPos(0.5 * (c.clipx - w), 0.5 * (c.clipy - w));
	}

	c.style = ERenderStyle.STY_Translucent;
	c.drawColor = 15 * hud.crosshairColor;
	
	if (w > 96 && xhair_xl != none) {
		c.drawTile(xhair_xl, w, w, 0, 0, 128, 128);
	} else {
		c.drawTile(xhair, w, w, 0, 0, 64, 64);
	}
}

function bool weaponSet(Pawn p) {
	if (bSlave) {
		return false;
	} else {
		super.weaponSet(p);
	}
}

function float switchPriority() {
	local int temp;

	if (bSlave) {
		return -10.0;
	}

	if (!owner.isA('PlayerPawn')) {
		return rateSelf(temp);
	}

	if (ammoLeft()) {
		if (slave != none) {
			return akimboSwitchPriority;
		}
	} else {
		if (Pawn(owner).weapon == self) {
			return -0.5;
		} else {
			return -1.0;
		}
	}
	
	return autoSwitchPriority;
}

function setSwitchPriority(Pawn p) {
	local int i;

	if (playerPawn(P) != none) {

		for (i = 0; i < arrayCount(PlayerPawn(p).weaponPriority); i++) {
			if (PlayerPawn(p).weaponPriority[i] == mimicPriorityOf) {
				autoSwitchPriority = i;
				return;
			}
		}
		
		super.setSwitchPriority(p);
	}
}

function setDisplayProperties(ERenderStyle style, Texture tex, bool bLighting, bool bEnviroMap) {
	if (!bSetUp) {
		bSetUp = true;
		if (slave != none) {
			slave.setDisplayProperties(style, tex, bLighting, bEnviroMap);
		}
		bSetUp = false;
	}
	
	super.setDisplayProperties(style, tex, bLighting, bEnviroMap);
}

function setDefaultDisplayProperties() {
	if (!bSetUp) {
		bSetUp = true;
		if (slave != none) {
			slave.setDefaultDisplayProperties();
		}
		bSetUp = false;
	}

	super.setDefaultDisplayProperties();
}

function bool handlePickupQuery(Inventory i) {
	local Pawn p;
	local int oldAmmo;
	local int rounds;

	if (i.class == class) {
		
		p = Pawn(owner);

		if (bAkimbo && slave == none) {
			
			slave = spawn(class, p);
			itemName = slaveName;
			AIRating = 0.4;
			slave.setUpSlave(self, Pawn(owner).weapon == self);
			
		} else {
			
			if (weapon(i).bWeaponStay && (!weapon(i).bHeldItem || weapon(i).bTossedOut)) {
				return true;
			}

			if (ammo().bClipped && SLWeapon(i).clip != none) {
				ammo().addClip(SLWeapon(i).clip);
			} else {
				if (ammoType != none) {
					oldAmmo = getAmmo();
					if (ammoType.addAmmo(weapon(i).pickupAmmoCount) && oldAmmo == 0 && p.weapon.class != i.class && !p.bNeverSwitchOnPickup) {
						weaponSet(p);
					}
				}
			}
			
		}

		p.receiveLocalizedMessage(class'PickupMessagePlus', 0, none, none, self.class);
		i.playSound(i.pickupSound);
		if (level.game.localLog != none) {
			level.game.localLog.logPickup(i, Pawn(owner));
		}

		if (level.game.worldLog != none) {
			level.game.worldLog.logPickup(i, Pawn(owner));
		}

		i.setRespawn();
		
		return true;
	}
	
	if (inventory == none) {
		return false;
	}

	return inventory.handlePickupQuery(i);
}

function setUpSlave(SLWeapon p, bool bBringUp) {
	becomeItem();
	
	master = p;
	bSlave = true;

	if (bFirearm) {
		fireMode = p.fireMode;
	}
	
	setDisplayProperties(p.style, p.texture, p.bUnlit, p.bMeshEnviroMap);
	giveAmmo(Pawn(owner));

	if (bBringUp) {
		bringUp();
	} else {
		gotoState('idle2');
	}
}

simulated function setHand(float hand) {
	if (bNoCenter && hand == 0) {
		hand = -1;
	}
	
	if (hand == 2) {

		bHideWeapon = true;
		playerViewOffset.y = 0.0;
		fireOffset.y = 0.0;

	} else {
		bHideWeapon = false;

		if (slave != none) {
			slave.setHand(-hand);
		}

		if (leftMesh != "" && hand == 1) {
			setSkelMesh(SkeletalMesh(dynamicLoadObject(leftMesh, class'SkeletalMesh')));
		} else {
			setSkelMesh(SkeletalMesh(default.playerViewMesh));
		}

		setPVO(hand, default.playerViewOffset);
		fireOffset.y = default.fireOffset.y * hand;
	}
}

simulated function setSkelMesh(SkeletalMesh m) {
	if (mesh != m) {
		skelAnim = none;
		mesh = m;
	}
}

simulated function setPVO(float hand, vector def) {
	local vector pvo;

	if (hand == 0) {
		pvo.x = def.x * 0.88;
		pvo.y = -0.2 * def.y;
		pvo.z = def.z * 1.12;
	} else {
		pvo.x = def.x;
		pvo.y = def.y * hand;
		pvo.z = def.z;
	}
	
	pvo *= 100;
	
	playerViewOffset = pvo;	
}

state clientActive {

	simulated function animEnd() {
		bBringingUp = false;

		if (!bSlave) {
			
			super.animEnd();
			
			if (slave != none && !isInState('ClientActive')) {
				if (getStateName() == 'None' || getStateName() == 'SLSidearm') {
					slave.gotoState('');
				} else {
					slave.gotoState(getStateName());
				}
			}
		}
	}

	simulated function beginState() {
		setHand(hand());

		super.beginState();

		bBringingUp = false;
		if (slave != none) {
			slave.gotoState('ClientActive');
		}
	}

}

state clientDown {

	simulated function animEnd() {
		if (!bSlave) {
			super.animEnd();
		}
	}

	simulated function endState() {
		if (slave != none) {
			slave.gotoState('');
		}
	}
  
}

function bringUp() {
	if (owner.isA('PlayerPawn')) {
		setHand(hand());
		PlayerPawn(owner).endZoom();
	}

	if (slave != none) {
		slave.bringUp();
	}

	bBringingUp = true;
	bWeaponUp = false;
	playSelect();
	gotoState('active');
}

simulated function slaveFire(bool bUseAlt, bool bWait) {
	bAlt = bUseAlt;

	if (bWait) {
		gotoState('slaveFireWait');
	} else {
		if (role < ROLE_Authority) {
			if (bAlt) {
				clientAltFire(0.0);
			} else {
				clientFire(0.0);
			}
		} else {
			if (bAlt) {
				altFire(0.0);
			} else {
				fire(0.0);
			}
		}
	}
}

simulated state slaveFireWait {

begin:
	if (bAlt) {
		sleep(slaveAltWait * frand());
	} else {
		sleep(slaveWait * frand());
	}
	
	slaveFire(bAlt, false);
}

simulated function remoteReload() {
	reload();
}

simulated function reload() {
	local SLAmmo a;

	a = ammo();
	
	if (a.bClipped) {

		if (mate() != none && mate().isInState('reloading') && a.getClips() == 1) {
			
			// Ignore.
			
		} else if (a.getClips() > 0) {
			
			if (ammoType != none && SLAmmo(AmmoType).spentClip != none) {
				spawnBrass(SLAmmo(ammoType).spentClip, 0.5);
			}
			
			playReload();
			
			gotoState('reloading');
			
			if (!u.hasViewPort(owner)) {
				remoteReload();
			}
			
		}
	}
}

simulated state reloading {

	ignores reload;
  
	function fire(float v) {
		if (slave != none) {
			slave.fire(v);
		}
	}
  
	function altFire(float v) {
		if (slave != none) {
			slave.altFire(v);
		}
	}
  
	simulated function bool clientFire(float v) {
		if (slave != none) {
			slave.clientFire(v);
		}
	}

	simulated function bool clientAltFire(float v) {
		if (slave != none) {
			slave.clientAltFire(v);
		}
	}
	
begin:
	finishAnim();
	
	if (bFakeReload) {
		playAss(assSelect);
		finishAnim();
	}
	
	if (role == ROLE_Authority) {
		clip = ammo().reload();
		finish();
	} else {
		gotoState('idle');
	}
}

simulated function playReload() {
	if (bFakeReload) {
		playAss(assDown);
	} else {
		playAss(assReload);
	}
}

simulated function bool playAss(ASS a, optional bool bReplicate, optional bool bSkipOwner) {
	local float v;

	if (a.sound != none && owner != none) {	
		v = a.volume * Pawn(owner).soundDampening;
		if (bReplicate) {
			replicateSound(a.sound, a.slot, v);
		} else if (bSkipOwner || a.bSkipOwner) {
			playOwnedSound(a.sound, a.slot, v);
		} else {
			playSound(a.sound, a.slot, v);
		}
	}

	if (a.aname != '' && a.arate > 0) {
		if (hasAnim(a.aname)) {
			playAnim(a.aname, a.arate, a.tween);
			return true;
		}
	}

	return false;
}

function replicateSound(Sound s, ESoundSlot slot, float vol) {
	playSound(s, slot, vol);
}

function fire(float v) {
	local bool bWait;

	if (ammoType == none) {
		giveAmmo(Pawn(owner));
	}
	
	if (ammoLeft()) {
		
		// This seems a suitable spot to allow the client to fire.
		bCanClientFire = true;
		
		if (needReload()) {
			
			reload();
			
		} else {
			
			bWait = true;
			if (bFirearm) {
				gotoState('FAFire');
			} else {
				fire2(v);
			}
			
		}
		
	} else {

		if (level.netMode == NM_Standalone || level.netMode == NM_ListenServer && u.hasViewPort(owner)) {
			playAss(assEmpty);
		}
	}

	if (slave != none) {
		slave.slaveFire(bAlt, bWait);
	}
}

function fire2(float v) {
	
	if (ammoLeft()) {
		if (bAlt) {
			gotoState('altFiring');
		} else {
			gotoState('normalFire');
		}

		bCanClientFire = true;
		bPointing = true;

		if (bRapidFire || firingSpeed > 0) {
			Pawn(owner).playRecoil(firingSpeed);
		}

		if (bInstantHit) {
			traceFire(0.0);
			useAmmo(1);
		} else {
			if (bAlt) {
				projectileFire(altProjectileClass, altProjectileSpeed, bWarnTarget);
			} else {
				projectileFire(projectileClass, projectileSpeed, bWarnTarget);
			}
		}
		
		if (bInstantHit || lastProj != none) {
			if (bAlt) {
				playAltFiring();
			} else {
				playFiring();
			}
			useAmmo(1);
		} else {
			finish();
		}
	}
}

function altFire(float v) {

	if (bAltSwitchMode) {
		switchMode();
		return;
	}
	
	bAlt = true;
	fire(v);
}

function switchMode() {
	playAss(assMode, true);
	switch (fireMode) {
    case FM_Single:
		fireMode = FM_Burst;
		break;
	case FM_Burst:
		fireMode = FM_Auto;
		break;
	case FM_Auto:
		fireMode = FM_Single;
		break;
	}

	if (slave != none) {
		slave.fireMode = fireMode;
	}
}

state idle {
	
	simulated function playIdleAnim() {
		if (!isAnimating() && frand() < idleRandom) {
			playAss(assIdle);
		}
	}
  
	function timer() {
		playIdleAnim();
	}
  
	simulated function beginState() {
		setTimer(idleLapse, true);
	}
  
	simulated function endState() {
		setTimer(0.0, false);
	}
	
begin:
	bPointing = false;
	
	if (!ammoLeft()) {
		Pawn(owner).switchToBestWeapon();
	}
	disable('AnimEnd');
}

state active {

	function beginState() {
		bChangeWeapon = False;
	}
  
	function endState() {
		super.endState();
		bBringingUp = false;
	}
  
	function bool putDown() {
		return super.putDown();
	}
	
begin:
	finishAnim();
	
	if (bChangeWeapon) {
		gotoState('DownWeapon');
	}

	bWeaponUp = true;
	bCanClientFire = true;
	
	if (!bSlave && level.netMode != NM_Standalone && owner != none && owner.isA('TournamentPlayer') && PlayerPawn(owner).player != none && !PlayerPawn(owner).player.isA('Viewport')) {
		if (Pawn(owner).bFire != 0) {
			TournamentPlayer(owner).sendFire(self);
			// Fixes the pseudo-fire bug
			global.fire(1.0);
		} else if (Pawn(owner).bAltFire != 0) {
			TournamentPlayer(owner).sendAltFire(self);
			//global.altFire(1.0);
		} else if (!bChangeWeapon) {
			TournamentPlayer(owner).updateRealWeapon(self);
		}
	}
	
	finish();
	
}

simulated function playFAFiring(optional bool bSkipNext) {
	if (bInitFA) {
		bInitFA = false;
	} else {
		if (fireMode == FM_Single) {
			playFiring();
		} else {
			PlayAltFiring();
		}
	}
	
	if (bSkipNext) {
		bInitFA = true;
	}
}

simulated state FAFire {
	ignores clientAltFire, altFire;

	function fire(float v) {		
		if (bClear) {
			gotoState('');
			global.fire(v);
		}
	}
  
	simulated function bool clientFire(float v) {
		if (bClear) {
			gotoState('');
			return global.clientFire(v);
		} else {
			return false;
		}
	}
  
	simulated function beginState() {
		bClear = false;
		burstCount = 0;
	}

begin:	
	// ?

fireLoop:
	while (((Pawn(owner).bFire != 0 || Pawn(owner).bAltFire != 0) && fireMode == FM_Auto) || (fireMode == FM_Burst && burstCount < burstRounds) || burstCount == 0) {
    	
    	if (needReload() || !ammoLeft()) {
			break;
		}
		
		bRapidFire = fireMode > 0;
		Pawn(Owner).playRecoil(firingSpeed);
		playFAFiring();
		
		if (role == ROLE_Authority) {
			if (level.netMode != NM_DedicatedServer) {
				spawnMuzzleFlash();
			}
			if (level.netMode != NM_Standalone) {
				flashCount++;
			}
			
			traceFire(vsize(recoilv) * recoilData.accScale);
			useAmmo(1);
		}
		
		recoil();
		spawnBrass(brassClass);
		sleep(60.0 / rpm);
		burstCount++;
	}

	sleep(burstWait);
	bClear = true;
	finishAnim();
	if (role == ROLE_Authority) {
		finish();
	} else {
		gotoState('idle');
	}
}

simulated function spawnMuzzleFlash() {
	local vector x, y, z, fo;

	if (owner != none) {
		fo = fireOffset * 2.0;
		getAxes(owner.rotation, x, y, z);
		spawn(class'SLMuzzleFlash', owner, , owner.Location + fo.x * x + fo.y * y * -hand() + fo.z * z);
		if (slave != none) {
			slave.spawnMuzzleFlash();
		}
	}
}

simulated function recoil() {
	local vector v;

	v = vrand() - vrand();
	v.z += frand() * recoilData.kick;
	recoilv += v * recoilData.base;
	recoilv.x = fclamp(recoilv.x, -recoilData.limit, recoilData.limit);
	recoilv.y = fclamp(recoilv.y, -recoilData.limit, recoilData.limit);
	recoilv.z = fclamp(recoilv.z, -recoilData.limit, recoilData.limit);
	
}

simulated function tick(float delta) {
	local Pawn p;
	local float x, y, z;
	local int dir;

	if (role < ROLE_Authority && flashCount > oldFlashCount) {
		spawnMuzzleFlash();
		oldFlashCount++;
	}

	p = PlayerPawn(owner);

	if (p != none) {
		if (vsize(recoilv) > 1.0) {
			recoilv = normal(recoilv) * fmax(vsize(recoilv) - recoilData.returnRate * delta, 0.0);
		}
	}
}

function traceFire(float acc) {
	if (slave != none || bSlave) {
		acc *= slaveAcc;
	}
	super.traceFire(acc);
}

function processTraceHit(Actor who, vector hitLoc, vector hitn, vector x, Vector y, Vector z) {
	if (who == level) {
		spawn(wallhitClass, who, , hitLoc + hitn, rotator(hitn));
	} else {
		if (who != self && who != owner && who != none) {
			who.takeDamage(hitdam, Pawn(owner), hitLoc, 3000.0 * x, myDamageType);
			if (!who.bIsPawn && !who.isA('Carcass')) {
				spawn(wallhitClass.default.puffClass, who, , hitLoc + hitn, rotator(hitn));
			} else {
				who.playSound(sound'ChunkHit', , 4.0, , 100.0);
			}
		}
	}
}

simulated function spawnBrass(class<SLEjectedBrass> c, optional float ejectv) {
	local Vector x;
	local Vector y;
	local Vector z;
	local SLEjectedBrass s;

	if (ejectv == 0) {
		ejectv = 1.0;
	}

	if (level.netMode != 1) {
		getAxes(owner.rotation, x, y, z);
		s = spawn(c, , , owner.location + calcDrawOffset() + 0 * x + -6 * z);
		if (s != none) {
			s.eject(brassv(x, y, z) * ejectv);
		}
	}
}

simulated function vector brassv(vector x, vector y, Vector z) {
	return ((frand() * 0.3 - 0.15) * x + (frand() * 0.2 + 0.0) * y * -hand() + (frand() * 0.3 + 0.1) * z) * 120;
}

function finish() {
	local Pawn po;
	local bool bForce;
	local bool bForceAlt;
	
	bForce = bForceFire;
	bForceAlt = bForceAltFire;
	bForceFire = false;
	bForceAltFire = false;
	bAlt = false;

	if (bChangeWeapon) {
		gotoState('downWeapon');
		return;
	}

	po = Pawn(owner);

	if (po == none) {
		return;
	}

	if (PlayerPawn(owner) == none) {
		
		if (!ammoLeft()) {
			po.stopFiring();
			gotoState('idle');
			po.switchToBestWeapon();

			if (bChangeWeapon) {
				gotoState('downWeapon');
			}

		} else if (po.bFire != 0 && frand() < refireRate) {
			global.fire(0.0);
		} else if (po.bAltFire != 0 && frand() < altRefireRate) {
			global.altFire(0.0);
		} else {
			po.stopFiring();
			gotoState('idle');
		}
		
	} else {
		tweenToStill();
		gotoState('idle');
	}
}

function dropFrom(vector startLoc) {
	if (slave == none) {
		if (ammo().bClipped) {
			ammoType = none;
			pickupAmmoCount = 0;
		}
		master = none;
		super.dropFrom(startLoc);
	} else {
		slave.dropFrom(startLoc);
	}
}

simulated function playFiring() {
	viewFX();
	playAss(assFire);
}

simulated function playAltFiring() {
	viewFX();
	playAss(assAltFire);
}

simulated function viewFX() {
	if (PlayerPawn(owner) != none && (level.netMode == NM_Standalone || PlayerPawn(owner).player.isA('Viewport'))) {
		if (instFlash != 0.0) {
			PlayerPawn(owner).clientInstantFlash(instFlash, instFog);
		}
		PlayerPawn(owner).shakeView(shaketime, shakemag, shakevert);
	}

	if (affector != none) {
		affector.fireEffect();
	}

	if (bDrawMuzzleFlash) {
		bMuzzleFlash++;
	}
}

simulated function bool clientFire(float v) {
	local bool bFired;

	if (bCanClientFire) {
		if (!ammoLeft()) {
			playAss(assEmpty);
		} else if (needReload()) {
			// Ignore.
		} else {
			if (bFirearm) {
				playFAFiring(true);
				gotoState('FAFire');
			} else {
				if (bAlt) {
					playAltFiring();
					gotoState('clientAltFiring');
				} else {
					playFiring();
					gotoState('clientFiring');
				}
			}
			
			bFired = true;
		}
		
		if (slave != none) {
			slave.slaveFire(bAlt, bFired);
		}
	}
	
	return bFired;
}

simulated function bool clientAltFire(float value) {
	if (bAltSwitchMode) {
		return false;
	}

	bAlt = true;
	return clientFire(value);
}

function Projectile projectileFire(class<Projectile> projClass, float speed, bool warn) {
	local vector start;
	local vector x, y, z;
	local Pawn po;

	lastProj = none;
	po = Pawn(owner);
	owner.makeNoise(po.soundDampening);
	getAxes(po.viewRotation, x, y, z);
	
	if (spawnProjOffset != vect(0.0, 0.0, 0.0)) {
		start = owner.location + spawnProjOffset;
	} else {
		start = owner.location + calcDrawOffset() + fireOffset.x * x + fireOffset.y * y + fireOffset.z * z;
	}
	
	adjustedAim = po.adjustAim(speed, start, aimError, true, warn);

	if (!bSetProjOwner) {
		po = none;
	}

	lastProj = spawn(projClass, po, , start, adjustedAim);

	return lastProj;
}

simulated function animEnd() {
	if (level.netMode == NM_Client && bBringingUp && mesh != pickupViewMesh) {
		bBringingUp = false;
		playSelect();
	} else {
		super.animEnd();
	}
}

state clientFiring {

	simulated function animEnd() {
		if (Pawn(owner) == none || !ammoLeft()) {
			playIdleAnim();
			gotoState('');
		} else if (!bCanClientFire) {
			gotoState('');
		} else {
			playIdleAnim();
			gotoState('');
		}
	}

}

state clientAltFiring {

	simulated function animEnd() {
		if (Pawn(Owner) == none || !ammoLeft()) {
			playIdleAnim();
			gotoState('None');
		} else if (!bCanClientFire) {
			gotoState('None');
		} else {
			playIdleAnim();
			GotoState('None');
		}
	}

}

simulated function playSelect() {
	bForceFire = false;
	bForceAltFire = false;
	bCanClientFire = false;

	if (!isAnimating() || animSequence != assSelect.aname) {
		playAss(assSelect);
	}
}

simulated function tweenDown() {
	if (isAnimating() && animSequence != '' && getAnimGroup(animSequence) == assSelect.aname) {
		tweenAnim(animSequence, animFrame * 0.4);
	} else {
		playAss(assDown);
	}
}

state downWeapon {
	ignores altFire, fire, animEnd;

	function beginState() {
		super.beginState();
		if (slave != none) {
			slave.gotoState('downWeapon');
		}
	}

}

simulated function renderOverlays(Canvas c) {
	local vector pvo;
	local PlayerPawn po;
	local float oldhand;

	if (owner == none) {
		return;
	}
	
	// Fix to get the postRender function working when UTPure is enabled.
	xPostRender(c);
	c.reset();

	if (bHideWeapon) {
		return;
	}
	
	po = PlayerPawn(owner);
	pvo = PlayerViewOffset;

	if (po != none) {
		pitchAdjust(po.rotation.pitch);
		PlayerViewOffset += recoilv * recoilData.pvoScale;
	}

	if (bDrawMuzzleFlash && arrayCount(MFVariations) > 0) {
		MFTexture = MFVariations[rand(arrayCount(MFVariations))];
	}

	if (po != none) {
		oldhand = po.handedness;
		po.handedness = hand();
	}

	super.renderOverlays(c);

	if (slave != none) {
		if (slave.bBringingUp) {
			slave.bBringingUp = false;
			slave.playSelect();
		}
		slave.renderOverlays(c);
	}

	if (po != none) {
		po.handedness = oldhand;
	}
	
	playerViewOffset = pvo;
}

simulated function pitchAdjust(int pitch) {
	if (pitch >= 49752) {
		pitch = max(62464, pitch);
	}

	if (pitch < 18000) {
		pitch = min(3072, pitch);
	}

	if (pitch > 3072) {
		pitch -= 65535;
	}

	playerViewOffset.x = 100.0 * (default.playerViewOffset.x + paSlide * pitch / 3072.0);

	if (bDrawMuzzleFlash) {
		flashY = default.flashY - paFlashY * pitch / 3072.0;
		flashO = default.flashO - paFlashO * pitch / 3072.0;
	}
}

simulated function SLWeapon mate() {
	if (bSlave) {
		return master;
	} else {
		return slave;
	}
}

simulated function float hand() {
	local float h;

	if (owner != none && owner.isA('PlayerPawn')) {
		h = PlayerPawn(owner).handedness;
	} else {
		h = -1;
	}

	if (bNoCenter && h == 0) {
		h = -1;
	}

	if (bSlave && h != 0) {
		h = -h;
	}

	return h;
}

function destroyed() {
	if (slave != none) {
		slave.destroy();
	}

	if (clip != none) {
		clip.destroy();
	}

	if (u != none) {
		u.Destroy();
	}

	super.destroyed();
}
// $END_CLASSIFIED

defaultproperties {
	brassClass=Class'SLEjectedBrass'
	wallhitClass=Class'SLWallhit'
	burstWait=0.05
	burstRounds=3
	recoilData=(Base=200.0,limit=1000.0,Kick=2.0,returnRate=1000.0,smoothRate=500.0,pvoScale=0.1,accScale=0.001),
	slaveWait=0.2
	slaveAltWait=0.2
	slaveAcc=2.0
	assFire=(aname=Fire,arate=1.00,tween=0.00,Sound=None,Volume=1.00,Slot=SLOT_None,bSkipOwner=True),
	assAltFire=(aname=AltFire,arate=1.00,tween=0.00,Sound=None,Volume=1.00,Slot=SLOT_None,bSkipOwner=True),
	assSelect=(aname=Select,arate=1.00,tween=0.00,Sound=None,Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assDown=(aname=Down,arate=1.00,tween=0.05,Sound=None,Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assReload=(aname=Reload,arate=1.00,tween=0.00,Sound=Sound'SLV2Sounds.Weapon.Reload',Volume=2.00,Slot=SLOT_Misc,bSkipOwner=False),
	assIdle=(aname=Idle,arate=1.00,tween=0.00,Sound=None,Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assEmpty=(aname=empty,arate=0.00,tween=0.00,Sound=Sound'SLV2Sounds.Weapon.empty',Volume=1.00,Slot=SLOT_Misc,bSkipOwner=False),
	assMode=(aname=empty,arate=0.00,tween=0.00,Sound=Sound'SLV2Sounds.Weapon.empty',Volume=0.50,Slot=SLOT_Misc,bSkipOwner=False),
	idleLapse=8.0
	idleRandom=0.1
	bNoCenter=True
	MFVariations(0)=Texture'SLV2Textures.muzf.mf0'
	MFVariations(1)=Texture'SLV2Textures.muzf.mf1'
	MFVariations(2)=Texture'SLV2Textures.muzf.mf2'
	MFVariations(3)=Texture'SLV2Textures.muzf.mf3'
	MFVariations(4)=Texture'SLV2Textures.muzf.mf4'
	MFVariations(5)=Texture'SLV2Textures.muzf.mf5'
	paSlide=2.0
	PickupAmmoCount=1
	bOwnsCrosshair=True
	Physics=PHYS_Falling
	bNoSmooth=False
}
