/***************************************************************************************************
 *
 *  TeleporterFix. CollisionHash server crash workaround by Zeropoint.
 *
 *  $CLASS        TeleporterFix
 *  $VERSION      1.02 (1-11-2008 14:46)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  An actor that fixes the teleporters in a map. The standard teleporters provided
 *                by the Unreal Engine have a nasty bug that could cause a server to crash on a
 *                collision hash check. This mutator replaces these teleporters with a custom
 *                version that does not suffer from this issue. It does not fix the bug, but
 *                prevents it from occurring. A big thanks should go to Casy Campbell who suggested
 *                the workaround. Thanks man, we owe you!
 *
 **************************************************************************************************/
class TeleporterFix extends Actor;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the teleporter fix. Iterates over all teleporters and fixes them.
 *
 **************************************************************************************************/
function postBeginPlay() {
	local Teleporter t;
	
	// For each teleporter in the game.
	foreach allactors(class'Teleporter', t) {
		// Fix it unless it is a TeleporterX, which does not need to be fixed.
		if (!t.isA('TeleporterX'))
			fixTeleporter(t);
	}
	
	// Fix complete, we no longer need this instance.
	destroy();
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Fixes the given teleporter. A custom teleporter will be spawned at the same
 *                location and the original teleporter will be disabled.
 *  $PARAM        t  The teleporter that is to be fixed.
 *  $REQUIRE      t != none
 *
 **************************************************************************************************/
function fixTeleporter(Teleporter t) {
	local Teleporter replacement;
	
	log("Fixing teleporter " $ t $ "...");
	
	// Disable original teleporter.
	t.setCollision(false, false, false);
	
	// Spawn replacement teleporter.
	replacement = spawn(class'TeleporterX', t.owner, t.tag, t.location, t.rotation);
	if (replacement == none)
		log("Failed to spawn replacement, teleporter won't work!");
	
	// Set replacement teleporter attributes.
	if (replacement != none) {
		// Copy settings from old teleporter.
		replacement.url = t.url;
		replacement.productRequired = t.productRequired;
		replacement.bChangesVelocity = t.bChangesVelocity;
		replacement.bChangesYaw = t.bChangesYaw;
		replacement.bReversesX = t.bReversesX;
		replacement.bReversesY = t.bReversesY;
		replacement.bReversesZ = t.bReversesZ;
		replacement.bEnabled = t.bEnabled;
		replacement.targetVelocity = t.targetVelocity;
		replacement.triggerActor = t.triggerActor;
		replacement.triggerActor2 = t.triggerActor2;
		replacement.lastFired = t.lastFired;
		replacement.setCollisionSize(t.collisionRadius, t.collisionHeight);
		
		// Make replacement teleporter invisible.
		replacement.bHidden = true;
		
		// Disable teleporter if it is a destination teleporter only.
		if (replacement.url ~= "")
        	replacement.setCollision(false, false, false);
	}
	
	// Clear tag of original teleporter.
	t.tag = 'DontUseMe';
	// Note that we can't destroy the teleporter without messing up the NavigationPoint list. Also
	// causes the AI to crash.
}