/***************************************************************************************************
 *
 *  TeleporterFix. CollisionHash server crash workaround by Zeropoint.
 *
 *  $CLASS        TeleporterX
 *  $VERSION      1.05 (9-12-2008 20:05)
 *  $AUTHOR       Daan 'Defrost' Scheerens  initial version
 *  $CONTACT      d.scheerens@gmail.com
 *  $DESCRIPTION  The custom teleporter that implements the workaround for the server crashes.
 *
 **************************************************************************************************/
class TeleporterX extends VisibleTeleporter;

struct ActorCollisionInfo {                       // Actor collision attributes structure
	var Actor a;                                  // The Actor whose attributes are stored.
	var bool bCollideActors;                      // Did the actor collide with other actors?
	var bool bBlockActors;                        // Did the actor block other actors?
	var bool bBlockPlayers;                       // Did the actor block PlayerPawns?
	var int targetTickCount;                      // Tick count at which the attributes should be
	                                              // retored.
};

struct ActorTouchIgnoreInfo {
	var Actor a;                                  // The actor whose touches are to ignored.
	var int targetTickCount;                      // Tick count at which touches from the actor
	                                              // should be allowed again.
};

var ActorCollisionInfo actorCollisionsToFix[10];  // Actors whose collision attributes have to be
                                                  // restored.
var ActorTouchIgnoreInfo touchIgnoreList[10];     // List of actors whose touches should be ignored.

var int tickCounter;                              // Counter for the number of elapsed ticks.

const tickCounterMax = 10;                        // Maximum value of the tick counter.
const restoreDelay = 3;                           // Number of ticks to wait before restoring
                                                  // collision attributes.
const ignoreDelay = 6;                            // Number of ticks to wait before allowing touches
                                                  // again from actors.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes the teleporter.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function postBeginPlay() {
	// Override VisibleTeleporter behaviour.
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called if an actor is to be teleported here. Before the actor is teleported it's
 *                collision is disabled. This should prevent the collision hash crash. A few ticks
 *                later the collision of the actor will be restored.
 *  $PARAM        incoming  The actor that is to be teleported.
 *  $PARAM        source    Source of the actor that is to be teleported.
 *  $REQUIRE      incoming != none && source != none
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function bool accept(Actor incoming, Actor source) {
	local rotator newRot, oldRot;
	local int oldYaw;
	local float mag;
	local vector oldDir;
	local pawn p;

	// Move the actor here.
	disable('Touch');

	newRot = incoming.rotation;
	if (bChangesYaw) {
		oldRot = incoming.rotation;
		newRot.yaw = rotation.yaw;
		if (source != none)
			newRot.yaw += (32768 + incoming.rotation.yaw - source.rotation.yaw);
	}

	if (Pawn(incoming) != none) {
		//tell enemies about teleport
		if (role == ROLE_Authority) {
			p = level.pawnList;
			while (p != none) {
    			if (p.enemy == incoming)
					p.lastSeenPos = incoming.location; 
				p = p.nextPawn;
			}
		}
		storeActor(incoming); // Collision fix.
		Pawn(incoming).setLocation(location);
		if ((role == ROLE_Authority) || (level.timeSeconds - lastFired > 0.5)) {
			newRot.yaw = rand(65536);
			Pawn(incoming).setRotation(newRot);
			Pawn(incoming).viewRotation = newRot;
			LastFired = level.timeSeconds;
		}
		Pawn(incoming).moveTimer = -1.0;
		Pawn(incoming).moveTarget = self;
		playTeleportEffect(incoming, false);
	} else {
		storeActor(incoming); // Collision fix.
		if (!incoming.setLocation(location)) {
			restoreActor(incoming); // Collision fix.
			enable('Touch');
			return false;
		}
		if (bChangesYaw)
			incoming.setRotation(newRot);
	}

	enable('Touch');

	if (bChangesVelocity) {
		incoming.velocity = targetVelocity;
	} else {
		if (bChangesYaw) {
			if (incoming.physics == PHYS_Walking)
			    oldRot.pitch = 0;
			oldDir = vector(oldRot);
			mag = incoming.velocity dot oldDir;     
			incoming.velocity = incoming.velocity - mag * oldDir + mag * vector(incoming.rotation);
		}
		if (bReversesX)
			incoming.velocity.x *= -1.0;
		if (bReversesY)
			incoming.velocity.y *= -1.0;
		if (bReversesZ)
			incoming.velocity.z *= -1.0;
	}   

	// Play teleport-in effect.

	return true;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called if an actor collides with this actor.
 *  $PARAM        other  The actor that touched this actor
 *  $REQUIRE      other != none 
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function touch(Actor other) {
	local int index;
	local bool bFound;
	
	// Check if the touch of this actor should be ignored.
	while (!bFound && index < arrayCount(touchIgnoreList)) {
		if (touchIgnoreList[index].a == other) {
			bFound = true;
		} else {
			index++;
		}
	}
	
	// Teleport if allowed.
	if (!bFound) {
		super.touch(other);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Disabled the collision for the specified actor and adds it to the list of actors
 *                whose collision attributes are to be restored.
 *  $PARAM        a  The actor whose collision is to be disabled (temporary).
 *  $REQUIRE      a != none
 *  $ENSURE       a.bCollideActors == false && a.bBlockActors == false && a.bBlockPlayers == false
 *
 **************************************************************************************************/
simulated function storeActor(Actor a) {
	local int index;
	local bool bFound;
	
	//broadcastMessage("Adding Actor " $ a $ " to the collision fix list...", true);
	
	while (!bFound && index < arrayCount(actorCollisionsToFix)) {
		if (actorCollisionsToFix[index].a == none) {
			actorCollisionsToFix[index].a = a;
			actorCollisionsToFix[index].bCollideActors = a.bCollideActors;
			actorCollisionsToFix[index].bBlockActors = a.bBlockActors;
			actorCollisionsToFix[index].bBlockPlayers = a.bBlockPlayers;
			actorCollisionsToFix[index].targetTickCount = (tickCounter + restoreDelay) % tickCounterMax;
			a.setCollision(false, false, false);
			bFound = true;
		} else {
			index++;
		}
	}
	
	addToTouchIgnoreList(a);
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Restores the collision attributes for the specified actor.
 *  $PARAM        a  The actor whose collision is to be restored.
 *  $REQUIRE      a != none
 *
 **************************************************************************************************/
simulated function addToTouchIgnoreList(Actor a) {
	local int index;
	local bool bFound;
	
	while (!bFound && index < arrayCount(touchIgnoreList)) {
		if (touchIgnoreList[index].a == none) {
			touchIgnoreList[index].a = a;
			touchIgnoreList[index].targetTickCount = (tickCounter + restoreDelay) % ignoreDelay;
			bFound = true;
		} else {
			index++;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Restores the collision attributes for the specified actor.
 *  $PARAM        a  The actor whose collision is to be restored.
 *  $REQUIRE      a != none
 *
 **************************************************************************************************/
simulated function restoreActor(Actor a) {
	local int index;
	local bool bFound;
		
	//broadcastMessage("Restoring Actor " $ a $ "...", true);
	
	while (!bFound && index < arrayCount(actorCollisionsToFix)) {
		if (actorCollisionsToFix[index].a == a) {
			a.setCollision(actorCollisionsToFix[index].bCollideActors,
			               actorCollisionsToFix[index].bBlockActors,
			               actorCollisionsToFix[index].bBlockPlayers);
			actorCollisionsToFix[index].a = none;
			bFound = true;
		} else {
			index++;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Timer tick. Checks if there are any actors for which the collision is to be
 *                restored.
 *  $PARAM        delta  The amount of time that has elapsed since the last tick call.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function tick(float delta) {
	local int index;
	
	tickCounter = (tickCounter + 1) % tickCounterMax;
	
	for (index = 0; index < arrayCount(actorCollisionsToFix); index++) {
		if (actorCollisionsToFix[index].a != none &&
		    actorCollisionsToFix[index].targetTickCount == tickCounter) {
			restoreActor(actorCollisionsToFix[index].a);
		}
	}
	
	for (index = 0; index < arrayCount(touchIgnoreList); index++) {
		if (touchIgnoreList[index].a != none &&
		    touchIgnoreList[index].targetTickCount == tickCounter) {
			touchIgnoreList[index].a = none;
		}
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Default properties block.
 *
 **************************************************************************************************/
defaultproperties {
	bCollideWhenPlacing=false
	drawType=DT_None
}

