/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLConfig
 *  $VERSION  1.03 (24-08-2010 13:43)
 *  $AUTHOR   Daan 'Defrost' Scheerens  original version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION  SLV Configuration container class. Also provides code to replicate some of the
 *                settings to the clients.
 *  $SINCE        SLV203
 *  $HISTORY      24-08-2010  Daan  Fixed replicated variables not being properly set on the client.
 *
 **************************************************************************************************/
class SLConfig extends ReplicationInfo config(SLV205XM);

enum EWarStatX { WH_Auto, WH_Armed, WH_Disarmed };
enum ESyncModeX { SN_NoSync, SN_ClientSync, SN_ServerSync };
enum EFireModeX { FM_Single, FM_Burst, FM_Auto };
enum ECarryMode { CM_Off, CM_Normal, CM_Compatability };

// Server settings.
var config int    maxRockets;           // Maximum amount of SLV rockets that can be carried.
var config float  teamDamage;           // Multiplier for damage caused by a teammate to SLV rockets.
var config int    initialFuelCores;     // Amount of fuel cores initially given to a player.
var config int    fuelCoresPerLauncher; // Fuel cores given to a player for each launcher picked up.
var config int    maxFuelCores;         // Max number of fuel cores to carry.
var config float  maxFuel;              // Default amount of fuel for rockets and cores.
var config bool   replacePacks;         // Replace health packs with fuel pods.
var config bool   replaceVials;         // Replace health vials with fuel pods.
var config bool   randomSLVPlacement;   // Spawn random SLV launchers over time.
var config bool   noYJ;                 // Remove the Yellow Jacket from the game.
var config bool   giveMaxAmmo;          // The SLV and YJ weapons get their maximum of ammo.
var config int    maxSidearmClips;      // Max number of YJ clips to carry.
var config int    roundsPerClip;        // Number of bullets per clip.
var config EFireModeX defSidearmMode;   // Default fire mode of the yellow jacket.
var config float  sidearmDamage;        // Hit damage caused by Yellow Jacket fire.
var config float  shockwaveSize;        // Size of the shockwave created by an exploding SLV rocket.
var config EWarStatX  warheadStatus;    // Initial warhead status of SLV rockets.
var config ESyncModeX syncMode;         // Rocket synchronisation method to use.
var config bool   noArmedSwitch;        // Players can't switch to armed warhead state.
var config bool   canOverrideWarStat;   // Players can't change the warhead state.
var config bool   useKillMsgFix;        // Use the kill message fix.
var config bool   killMsgFixSLVOnly;    // Limit the kill message fix to SLV rockets only.
var config float  minSpeed;             // Rocket velocity at minimum throttle.
var config float  fullSpeed;            // Rocket velocity at maximum throttle.
var config float  maxSpeed;             // Rocket velocity when the afterburner is enabled.
var config float  rocketArmor;          // Amount of damage the rocket hull can withstand.
var config bool   enableBoosters;       // Enable the use of boosters for SLV rockets.
var config float  boosterFuel;          // Amount of fuel for the boosters.
var config float  boosterSpeed;         // Extra speed gained by using the boosters.
var config bool   coloredDetonations;   // Use team colored detonations.
var config bool   noPilotTeamDamage;    // Do unpiloted slv rockets take team damage?
var config string excludedActors[16];   // Actors excluded from being replaced by StrangeArena.
var config string customSLVSkins[8];    // Customized rocket skins.
var config ECarryMode pilotCarryMode;   // Is the pilot of a SLV rocket allowed to carry the flag?

// Client settings.
var config bool  showKeys;              // Display an overview of the SLV controls.
var config bool  showRearView;          // Show the rear view mirror when flying on a rocket.
var config bool  disableContrails;      // Don't show the rocket contrails.
var config bool  contrailRotate;        // Contrail sections slowly rotate.
var config bool  contrailViewAlign;     // Contrail sections automatically align to the players view.
var config float contrailLifeTime;      // Life time of the contrail sections.
var config bool  disableLaunchScream;   // Disable the scream sound played when launching a rocket.
var config bool  showDamageSparks;      // Spawn spark particles when a SLV rocket takes damage.

// Replication checking.
var string replicationCheck;            // Used to detect when the initial replication is complete.
const replicationSignal = "NetReady";   // Value that signals completion of the initial replication.
const replicationCheckFreq = 20.0;      // Frequency of the replication checks.


// Replication buffer.
var int    _maxFuelCores;         // Max number of fuel cores to carry.
var float  _maxFuel;              // Default amount of fuel for rockets and cores.
var int    _roundsPerClip;        // Number of bullets per clip.
var float  _shockwaveSize;        // Size of the shockwave created by an exploding SLV rocket.
var EFireModeX _defSidearmMode;   // Default fire mode of the yellow jacket.
var EWarStatX  _warheadStatus;    // Initial warhead status of SLV rockets.
var ESyncModeX _syncMode;         // Rocket synchronisation method to use.
var float  _minSpeed;             // Rocket velocity at minimum throttle.
var float  _fullSpeed;            // Rocket velocity at maximum throttle.
var float  _maxSpeed;             // Rocket velocity when the afterburner is enabled.
var float  _rocketArmor;          // Amount of damage the rocket hull can withstand.
var bool   _enableBoosters;       // Enable the use of boosters for SLV rockets.
var float  _boosterFuel;          // Amount of fuel for the boosters.
var float  _boosterSpeed;         // Extra speed gained by using the boosters.
var string _customSLVSkins[8];    // Customized rocket skins.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Replication block.
 *
 **************************************************************************************************/
replication {
	reliable if (role == ROLE_Authority && bNetInitial)
		_maxFuelCores, _maxFuel, _roundsPerClip, _shockwaveSize, _warheadStatus, _syncMode,
		_minSpeed, _fullSpeed, _maxSpeed, _rocketArmor, _enableBoosters, _boosterFuel,
		_boosterSpeed, _defSidearmMode, _customSLVSkins, replicationCheck;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Copies the replicated server settings to or from the replication buffer.
 *  $PARAM        bFromBuffer whether or the settings are to be copied from the buffer.
 *
 **************************************************************************************************/
simulated function copyReplicatedSettings(optional bool bFromBuffer) {
	local int index;
	
	if (bFromBuffer) {
		maxFuelCores          = _maxFuelCores;
		maxFuel               = _maxFuel;
		roundsPerClip         = _roundsPerClip;
		shockwaveSize         = _shockwaveSize;
		warheadStatus         = _warheadStatus;
		syncMode              = _syncMode;
		minSpeed              = _minSpeed;
		fullSpeed             = _fullSpeed;
		maxSpeed              = _maxSpeed;
		rocketArmor           = _rocketArmor;
		enableBoosters        = _enableBoosters;
		boosterFuel           = _boosterFuel;
		boosterSpeed          = _boosterSpeed;
		defSidearmMode        = _defSidearmMode;
		for (index = 0; index < arrayCount(customSLVSkins); index++)
		customSLVSkins[index] = _customSLVSkins[index];
	} else {
		_maxFuelCores          = maxFuelCores;
		_maxFuel               = maxFuel;
		_roundsPerClip         = roundsPerClip;
		_shockwaveSize         = shockwaveSize;
		_warheadStatus         = warheadStatus;
		_syncMode              = syncMode;
		_minSpeed              = minSpeed;
		_fullSpeed             = fullSpeed;
		_maxSpeed              = maxSpeed;
		_rocketArmor           = rocketArmor;
		_enableBoosters        = enableBoosters;
		_boosterFuel           = boosterFuel;
		_boosterSpeed          = boosterSpeed;
		_defSidearmMode        = defSidearmMode;
		for (index = 0; index < arrayCount(customSLVSkins); index++)
		_customSLVSkins[index] = customSLVSkins[index];
	}
		
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Initializes this object. Applies the settings for client or server.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function preBeginPlay() {
	if (role == ROLE_Authority) { 
		applyServerSettings(); 
		applyReplicatedServerSettings();
		copyReplicatedSettings();
		replicationCheck = replicationSignal;
	} 
	
	if (level.netMode != NM_DedicatedServer) { 
		applyClientSettings(); 
	}
	
	if (level.netMode == NM_Client) { 
		setTimer(1.0 / replicationCheckFreq, true);
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Timer tick. During each tick the client checks if the replicated server variables
 *                have been received. If this is the case the settings are applied and the timer is
 *                stopped.
 *  $OVERRIDE
 *
 **************************************************************************************************/
simulated function timer() {
	// Check if initial replication info has been received.
	if (replicationCheck == replicationSignal) {
		setTimer(0.0, false);
		copyReplicatedSettings(true);
		applyReplicatedServerSettings();
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Applies the server settings.
 *
 **************************************************************************************************/
function applyServerSettings() {
	class'StrangeAmmo'.default.configMaxAmmo = maxRockets;
	class'SLSidearmAmmo'.default.maxClips = maxSidearmClips;
	class'SLSidearm'.default.hitDam = sidearmDamage;
	class'StrangeShell'.default.teamDamage = teamDamage;
	class'StrangeShell'.default.bDisableArmedWH = noArmedSwitch;
	class'StrangeShell'.default.bCanOverride = canOverrideWarStat;
	class'SLKillMsgFix'.default.bEnableSLVCheck = killMsgFixSLVOnly;
	class'StrangeShell'.default.bTeamDetClass = coloredDetonations;
	class'StrangeShell'.default.bNoPilotTeamDam = noPilotTeamDamage;
	class'StrangeShell'.default.bPilotCantCarryFlag = pilotCarryMode != CM_Off;
	class'SLKongLauncher'.default.numFuelCoresOnPickup = fuelCoresPerLauncher;
	class'SLFuelcore'.default.configMaxAmmo = maxFuelCores;
	class'SLFuelcore'.default.maxAmmo = maxFuelCores;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Applies the server settings which the client also has to know.
 *  $REQUIRE      replicationCheck == replicationSignal
 *
 **************************************************************************************************/
simulated function applyReplicatedServerSettings() {
	default.maxFuelCores = maxFuelCores;
	default.defSidearmMode = defSidearmMode;
	default.roundsPerClip = roundsPerClip;
	default.shockwaveSize = shockwaveSize;
	default.syncMode = syncMode;
	default.maxFuel = maxFuel;
	default.warheadStatus = warheadStatus;
	default.minSpeed = minSpeed;
	default.fullSpeed = fullSpeed;
	default.maxSpeed = maxSpeed;
	default.boosterSpeed = boosterSpeed;
	default.enableBoosters = enableBoosters;
	default.boosterFuel = boosterFuel;
	default.boosterSpeed = boosterSpeed;
	default.rocketArmor = rocketArmor;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Applies the client settings.
 *
 **************************************************************************************************/
simulated function applyClientSettings() {
}

// $END_CLASSIFIED

defaultproperties {
	netPriority=8.0
	remoteRole=ROLE_SimulatedProxy
	maxRockets=6
	teamDamage=0.0
	initialFuelCores=0
	fuelCoresPerLauncher=0
	maxFuelCores=5
	maxFuel=300.0
	replacePacks=false
	replaceVials=false
	randomSLVPlacement=false
	noYJ=false
	giveMaxAmmo=false
	maxSidearmClips=10
	roundsPerClip=60
	sidearmDamage=7
	defSidearmMode=FM_Single
	shockwaveSize=10.0
	warheadStatus=WH_Auto
	syncMode=SN_ServerSync
	noArmedSwitch=false
	showKeys=true
	showRearView=false
	disableContrails=false
	contrailRotate=false
	contrailViewAlign=false
	contrailLifeTime=20.0
	disableLaunchScream=false
	canOverrideWarStat=true
	useKillMsgFix=true
	killMsgFixSLVOnly=true
	minSpeed=500.0
	fullSpeed=900.0
	maxSpeed=1350.0
	rocketArmor=100.0
	enableBoosters=false
	boosterFuel=40.0
	boosterSpeed=400.0
	coloredDetonations=false
	showDamageSparks=false
	noPilotTeamDamage=false
	pilotCarryMode=CM_Off
}