/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLReplace
 *  $VERSION  1.00
 *  $AUTHOR   Daan 'Defrost' Scheerens  original version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/

// $CLASSIFIED

/***************************************************************************************************
 *
 *  $DESCRIPTION  Mutator to replace embedded SLVs, Yellow Jackets and Fuel Cores from older
 *                StrangeLove versions with those of the current version.
 *  $SINCE        SLV203
 *
 **************************************************************************************************/
class SLReplace extends Mutator;



/***************************************************************************************************
 *
 *  $DESCRIPTION  Checks whether the given actor should be replaced by another one, and does so if
 *                this is the case.
 *  $PARAM        other           The actor for which has to be checked if it is to be replaced.
 *  $PARAM        bSuperRelevant  Indicates if this actor is always relevant for the game.
 *  $REQUIRE      other != none
 *  $RETURN       True if the actor is relevant, false if it is not. In case the actor is replaced
 *                false should be returned. False may also be returned if the actor is not replaced,
 *                but shouldn't appear in the game.
 *
 **************************************************************************************************/
function bool checkReplacement(Actor other, out byte bSuperRelevant) {
	local bool bRelevant;

	bRelevant = true;
	
	if (bRelevant) bRelevant = !upgradeActor(other, 'SLKongLauncher', 'KongLauncher'); // For SLV2.
	if (bRelevant) bRelevant = !upgradeActor(other, 'SLKongLauncher'); // For SLV2xx.
	
	if (bRelevant) bRelevant = !upgradeActor(other, 'SLSideArm', 'SideArm');
	if (bRelevant) bRelevant = !upgradeActor(other, 'SLSideArm');
	
	if (bRelevant) bRelevant = !upgradeActor(other, 'SLFuelCore', 'FuelCore');
	if (bRelevant) bRelevant = !upgradeActor(other, 'SLFuelCore');
	
	return bRelevant;
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Upgrades the given actor to that of the specified target class and package, if the
 *                class of the actor matches oldClass. In case oldClass is not specified,
 *                targetClass will be used instead. If targetPackage is omitted, the target package
 *                will be the package that contains the class where this function is located.
 *  $PARAM        other          The actor that possibly has to be upgraded to another version.
 *  $PARAM        targetClass    Name of the class for the replacement actor.
 *  $PARAM        oldClass       Name of the class for the actor that should be replaced.
 *  $PARAM        targetPackage  Package name in which the replacement class is defined.
 *  $REQUIRE      other != none && targetClass != ''
 *  $RETURN       True if the given actor was upgraded, false if not.
 *
 **************************************************************************************************/
function bool upgradeActor(Actor other, name targetClass, optional name oldClass,
						   optional string targetPackage) {
	
	if (oldClass == '') {
		oldClass = targetClass;
	}
	
	// Should this class be replaced?
	if (!other.isA(oldClass)) {
		return false;
	}
	
	// Get target package.
	if (targetPackage == "") {
		targetPackage = left(string(self.class), instr(string(self.class), "."));
	}
	
	// Check package.
	if (left(string(other.class), len(targetPackage) + 1) != (targetPackage $ ".")) {
		replaceWith(other, targetPackage $ "." $ targetClass);
		return true;
	} else {
		return false;
	}
}



/***************************************************************************************************
 *
 *  $DESCRIPTION  Spawns a replacement for the given actor of the specified class.
 *  $PARAM        other       The actor for which a replacement should be spawned.
 *  $PARAM        aClassName  Class of the instance that should replace the actor.
 *  $REQUIRE      other != none
 *  $RETURN       True if a replacement actor has been spawned, false if this action failed.
 *
 **************************************************************************************************/
function bool replaceWith(Actor other, string aClassName) {
	local Actor a;
	local class<Actor> aClass;
	
	// Get class of replacement actor.
	aClass = class<Actor>(dynamicLoadObject(aClassName, class'Class'));
	
	// Spawn replacement actor.
	if (aClass != none) {
		a = spawn(aClass, other.owner, other.tag, other.location, other.rotation);
	}
	
	// Copy properties.
	if (a != none) {
		a.ambientGlow = other.ambientGlow;
		a.drawScale = other.drawScale;
		a.scaleGlow = other.scaleGlow;
		a.event = other.event;
		a.tag = other.tag;
		a.desiredRotation = other.desiredRotation;
		a.mass = other.mass;
		a.rotationRate = other.rotationRate;
		a.velocity = other.velocity;
		return true;
	} else {
		return false;
	}
}

// $END_CLASSIFIED
