/***************************************************************************************************
 *
 *  SLV2. Original version by 0Fus. Maintained since version 2.02 by Zeropoint.
 *
 *  $CLASS    SLFlagSN
 *  $VERSION  1.05
 *  $AUTHOR   Daan 'Defrost' Scheerens  original version
 *  $CONTACT  d.scheerens@gmail.com
 *
 **************************************************************************************************/
 
// $CLASSIFIED

class SLFlagSN extends SpawnNotify;

var() class<CTFFlag> toReplace[4];      // Original flag class to replace for each team.
var() class<CTFFlag> replacement[4];    // Replacement flag class for each team.



/***************************************************************************************************
 *
 *  $DESCRIPTION  Called when an actor of the specified type (actorClass) is spawned. Checks if the
 *                spawned actor is a flag that should be replaced and does so if this is true.
 *  $PARAM        a  The actor that was spawned.
 *  $REQUIRE      a != none
 *  $RETURN       The Actor that was spawned or a new Actor if the old one was replaced.
 *  $ENSURE       result != none
 *
 **************************************************************************************************/
simulated event Actor spawnNotification(Actor a) {
	local Actor newActor;
	local Actor flagOwner;
	local vector flagLocation;
	local rotator flagRotation;
	local class<Actor> aClass;
	local FlagBase fb;
	local int team;
	local bool bReplace;
	
	// Check if the actor is a flag, has to be replaced and determine the team.
	if (a.isA('CTFFlag')) {
		while (!bReplace && team < arrayCount(toReplace)) {
			if (a.class == toReplace[team]) {
				bReplace = true;
			} else {
				team++;
			}
		}
	}
	
	// Replace the flag?
	if (bReplace) {
		// Yes, destroy the original flag and spawn a new one.
		
		// Save attributes from original flag.
		flagOwner = a.owner;
		flagLocation = a.location;
		flagRotation = a.rotation;
		
		// Spawn new flag.
		newActor = spawn(replacement[team], flagOwner, , flagLocation, flagRotation);
		
		// Check if new flag has been spawned.
		if (newActor == none) {
			// It hasn't been spawned, use the original flag.
			warn("Failed to replace flag " $ a);
			newActor = a;
			
		} else {
			// Get flag base.
			foreach AllActors(class'FlagBase', fb) {
				if (fb.team == team) {
					break;
				} else {
					fb = none;
				}
			}
			
			// Replacement flag has been spawned, destroy the original flag.
			log("Flag " $ a $ " has been replaced by " $ newActor);
			CTFFlag(a).homeBase = fb; // To prevent accessed none warning in CTFFlag.home.EndState.
			a.setCollision(false, false, false);		
			a.destroy();
			
			// Update flag base.
			if (fb != none) {
				fb.bUnlit = true;
				fb.bHidden = false; // Is set to true by the old flag when it is destroyed.
			}
			
		}
		
	} else {
		// No, return the original actor.
		newActor = a;
		
	}
	
	return newActor;
}

// $END_CLASSIFIED

defaultproperties {
    actorClass=class'Botpack.CTFFlag'
    toReplace(0)=class'Botpack.RedFlag'
    toReplace(1)=class'Botpack.CTFFlag'
    replacement(0)=class'SLFlagRed'
    replacement(1)=class'SLFlag'
}